angular.module('calories', [])

.controller('CaloriesController', function($scope, localStorageService, stats, $timeout, goals, $ionicScrollDelegate, weekOverview, monthOverview, generalOverview, yearOverview, dayOverview, dayOverview_options, $translate) {
     
    // vars
    $scope.active = {};
    $scope.stats = {};
    $scope.activeOverview;
    $scope.overview = {
        options: {},
        data: [],
        labels: [],
        date:moment().format('dd, DD.MM.YY'),
        limit: false
    }
    
    var setStats = function () {
        stats.refresh();
        $scope.stats = stats.recursive(); 
    };
    setStats();   
    
    // (re-)initialize values on entering
    $scope.$on('$ionicView.enter', function() {
        $scope.controls(0);
        daytimes();
    });
    
    // when today is displayed and new data comes in
    $scope.$on('newData', function(event, args){
        if (args.type == 'daytime' || args.type == 'hour') {
            if ($scope.hour_selected == moment().format("YYYY-MM-DD")) {
                dayOverview.refresh();
                weekOverview.refresh();
                monthOverview.refresh();
                yearOverview.refresh();
                
                $scope.bar_setActive(0);
                $scope.controls(days);
            }
        };
        daytimes();
    });
    
    // chart defaults
    $scope.hour_options = dayOverview_options.options();
    $scope.hour_selected = moment().format('YYYY-MM-DD');
    $scope.hour_options.legend['onClick'] = function(e,p){
        var tmpBool = $scope.hour_datasetOverride[p.datasetIndex].hidden;
        $timeout(function(){
            $scope.hour_options.scales.yAxes[p.datasetIndex].ticks.display = tmpBool;
            $scope.hour_datasetOverride[p.datasetIndex].hidden = !tmpBool;
             if ($scope.week_options) {
                 $scope.week_options.scales.yAxes[p.datasetIndex].ticks.display = tmpBool;
             }
        });
    }
    $scope.hour_datasetOverride = [{ 
        yAxisID: 'y-axis-1', 
        hidden:true, 
        fill:true,
        lineTension:0.1
    }, { 
        yAxisID: 'y-axis-2',
        hidden:false, 
        fill:true,
        lineTension:0.1
    }];
    
    /*
     *  VERLAUF
     */
    
   function daytimes() {
        var tmp_daytimes = [];
        var moving_data = localStorageService.get('moving_data') || {};
        for (key in moving_data) {
            if (moving_data.hasOwnProperty(key)) {
                var tmp = moving_data[key];
                  var _tmp = tmp[tmp.length-1];
                _tmp['date'] = key;
                
                if (_tmp.s != 0 || _tmp.d != 0 || _tmp.c != 0) tmp_daytimes.push(tmp[tmp.length-1]);
            }
        }
        $scope.daytimes = tmp_daytimes;
    }
    
    
    $scope.jumpTo = function (value) {
        $scope.bar_setActive(0);
        var target = moment(value.date);
        var active = moment($scope.hour_selected);
        var days = target.diff(active, 'days');
        $scope.controls(days);
        $ionicScrollDelegate.scrollTop(true);
    }
    
   /*
     *  switch between day, week, month and year
     */
    
    $scope.bar_active = 0;
    $scope.bar_setActive = function(type) {
        $scope.bar_active = type;
        $scope.switchDataType (type); 
    };
    $scope.bar_isActive = function(type) {
        return type === $scope.bar_active;
    };
    
    $scope.switchDataType = function(type) {
        switch (type) {
            case 0:     
                $scope.hour_colors = ['#89cc2e', '#e1e100'];
                $scope.hour_series = [$translate.instant('Schritte'), $translate.instant('Kalorien')];
                $scope.activeOverview = dayOverview;
                break;
            case 1:
                $scope.hour_colors = ['#e1e100'];    
                $scope.hour_series = [$translate.instant('Kalorien')];
                $scope.activeOverview = weekOverview;
                break;
            case 2:
                $scope.hour_colors = ['#e1e100'];     
                $scope.hour_series = [$translate.instant('Kalorien')];
                $scope.activeOverview = monthOverview;
                break;
            case 3:
                $scope.hour_colors = ['#e1e100'];    
                $scope.hour_series = [$translate.instant('Kalorien')]; 
                $scope.activeOverview = yearOverview;
                break;
        }
        switchToOverview();
    }
    $scope.bar_setActive(0);
    
    /*
     *  OVERVIEW 
     */
    
    $scope.controls = function(add) {
        $scope.activeOverview.change(add);
        switchToOverview();
    }
    
    function activeData (totalData, count, monthsData) {
        var oActive = totalData;
        if ($scope.bar_active == 0) {
            // day
            oActive['goal'] = goals.type('calories');
            oActive['percentage'] = Math.floor(oActive.calories / oActive.goal * 100);
        } else {
            if ($scope.bar_active != 3) {
                // week/month
                oActive['goal'] = count * goals.type('calories');
            } else {
                // year
                oActive['goal'] = 0;
                for (var i = 0; i < monthsData.length; i++) {
                    var tmpGoal = monthsData[i] * goals.type('calories');
                    oActive.goal += tmpGoal;
                }
            }
            var perc;
            if (oActive.goal > 0) perc = Math.floor(oActive.calories / oActive.goal * 100);
            else {
                perc = 0;
                oActive.goal = goals.type('calories');
            }
            oActive['percentage'] = perc;
        }
        return oActive;
    }
    
    function switchToOverview () {
        // fetch options if needed
        if (Object.keys($scope.overview.options).length == 0) 
            $scope.overview.options = generalOverview.options();
        
        // set title
        if ($scope.overview.options.title.text != $scope.activeOverview.title()) {
            $scope.overview.options.title.text = $scope.activeOverview.title();
            $scope.overview.options.scales.xAxes[0] = $scope.activeOverview.override();
        }
        
        // get new data
        $scope.activeOverview.data(true).then(function(data) {
            $scope.overview.data = data.data;
            $scope.overview.labels = data.labels;
            $scope.overview.date = data.date;
            $scope.overview.limit = data.limit;
            $scope.active = activeData(data.total, data.count, data.monthsData);
        });
    }
});