angular.module('starter.controllers', ['sleep','btmessage','btconnect']) // TODO: btmessage, sleep only for testing
  
.controller('MenuCtrl', function($scope, $translate, bt_sms_messenger, mambo2_address, localStorageService, $mdDialog, $ionicPlatform) {
    $scope.isIOS = ionic.Platform.isIOS();
    $scope.isAndroid = ionic.Platform.isAndroid();
    
    $translate('LINK FAQ').then(function (translation) {
        $scope.link = translation;
        $scope.externalLink = function () {
            window.open($scope.link,'_system', 'location=true');
        }
    });
    
    
    /*
     *  BLUETOOTH INDICATOR
     */

    $scope.btIndicatorState = 2;
    $scope.$on("btConnection", function(event, args) {
        $scope.btIndicatorState = args.state;
    });
    
    
    /* 
     * BATTERY 
     */
    
    $scope.batteryClass = "";
    $scope.batteryLevel = function(force) {
        var battery = localStorageService.get("battery") || {newVal:{level:7},oldVal:{level:7}};
        if ( battery.newVal.level !== battery.oldVal.level  || force) {
             switch (battery.newVal.level) {
                case 0:
                    $scope.batteryClass = "ion-battery-empty low-battery";
                    break;
                case 1:
                case 2:
                    $scope.batteryClass = "ion-battery-low";
                    break;
                case 3:
                case 4:
                    $scope.batteryClass = "ion-battery-half";
                    break;
                default:
                    $scope.batteryClass = "ion-battery-full";
                    break;

            } 
            if (battery.newVal.level < 1 && battery.oldVal.level >= 1) {
                batteryCritical();
            }
        };
    };
    
    $scope.$watch('newData', function() {
        $scope.batteryLevel();
    });
    
    $scope.$on('$ionicView.enter', function() {
        $ionicPlatform.ready(function () {
            $scope.batteryLevel(true);
        });
    });
    
    function batteryCritical() {
        var alert = $mdDialog.alert({
            title: $translate.instant('Batteriestand niedrig'),
            textContent: $translate.instant('TEXT Batteriestand niedrig'),
            ok: $translate.instant('verstanden')
        });
        $mdDialog.show( alert );
    }
})

.controller('SettingsCtrl', function($scope, $stateParams) { 
})

.controller('MainController', function($scope, $timeout, $translate, $filter, $mdDialog, goals, dashboard, $state, localStorageService, $stateParams, $rootScope, btinterface,$ionicPlatform, bthf_init, motivational) {
     var initValues = function(refresh) {
        if (refresh) dashboard.refresh();
        var currentData = dashboard.values();

        var steps = {
            index:0,
            goal: goals.type("steps"),
            current: currentData.s,
            imagezoom: 100,
            aria: $translate.instant('Schritte'),
            type: "steps",
            zoom: .8,
            extra: "dark left-10"
        }
        steps['perc'] = steps.current/steps.goal;
        var _steps_mot = motivational.getMot('steps', steps.perc);
        if (_steps_mot) steps['mot'] = _steps_mot;
        else steps['mot'] = $scope.categories[0].mot;

        var calories = {  
            index:2, 
            goal: goals.type("calories"),
            current: currentData.c,
            text: "kcal",
            aria: $translate.instant('Kalorien'),
            type: "calories",
            zoom: .8,
            extra: "dark right-50"
        }
        calories['perc'] = calories.current/calories.goal;
        var _calories_mot = motivational.getMot('calories', calories.perc);
        if (_calories_mot) calories['mot'] = _calories_mot;
        else calories['mot'] = $scope.categories[2].mot;

        var heart = {
            index:1,
            current: currentData.h,
            goal: goals.type("heart"),
            imagezoom: 90,
            aria: $translate.instant('Puls'),
            type: "heart",
            normalize: true,
            zoom: .8,
            extra: "dark right-50"
        }
        heart['perc'] = heart.current/heart.goal;
        var _heart_mot = motivational.getMot('heart', heart.perc);
        if (_heart_mot) heart['mot'] = _heart_mot;
        else heart['mot'] = $scope.categories[1].mot;

        if (!$scope.categories[0]) {
            var sleep = {  
                index:3, 
                goal: goals.type("sleep"),
                current: 0,
                imagezoom: 80,
                aria: $translate.instant('Schlaf'),
                type: "sleep",
                fakeCurrent: $filter('numbertotime')(0),
                zoom: .8,
                extra: "dark left-10"
            }
            sleep['perc'] = sleep.current/sleep.goal;

            $scope.categories = [steps, heart, calories, sleep];
        } else {
           $scope.categories[0] = steps; 
           $scope.categories[1] = heart; 
           $scope.categories[2] = calories; 
        }

        // async

        // sleep 
        dashboard.async('sleep').then(function(sleepVal){
            var pretty_sleep = pad(Math.floor(sleepVal/60))+":"+pad(Math.floor(sleepVal%60));
            var mot = $scope.categories[3]['mot'];
            $scope.categories[3] = {  
                index:3, 
                goal: goals.type("sleep"),
                current: sleepVal/60,
                imagezoom: 80,
                aria: $translate.instant('Schlaf'),
                type: "sleep",
                fakeCurrent: pretty_sleep,
                zoom: .8,
                extra: "dark left-10"
            };
            $scope.categories[3]['perc'] = $scope.categories[3].current/$scope.categories[3].goal;
            var _sleep_mot_async = motivational.getMot('sleep', $scope.categories[3].perc);
            if (_sleep_mot_async) $scope.categories[3]['mot'] = _sleep_mot_async;
            else $scope.categories[3]['mot'] = mot;
            
            if ($scope.active.index == 3) 
                $scope.active = angular.copy($scope.categories[3]);
        });
    }
    
    $scope.$on('$ionicView.enter', function() {
        var useHealth = localStorageService.get('useHealth') || {val:false};
        if (useHealth.val) bthf_init.init();
        
        
        $ionicPlatform.ready(function () {
            if ($rootScope.myBleIsConnected) $scope.btIndicatorState = 1;
            else $scope.btIndicatorState = 2;

            initValues(true);
            $scope.active = angular.copy($scope.categories[$scope.active.index]);
        });
    });
    
    $ionicPlatform.ready(function () {

        $scope.categories = [];
        $scope.zoom = 0.25;

        initValues();

        $scope.active = angular.copy($scope.categories[0]);

        $scope.catSelect = function (cat) {
            $scope.active.current = 0;
            $scope.active.target = 0;
            $timeout (function() {
                $scope.active = angular.copy(cat);

                $scope.setMotivation(cat);
            },10);
        }


        $scope.setMotivation = function(cat){
                $scope.perc = Math.floor(cat.current / cat.goal * 100);
                $scope.total = cat.fakeGoal ? cat.fakeGoal : cat.goal;
                if ($scope.perc < 50) $scope.motivation = "DASH_MOT_"+$filter('uppercase')(cat.type)+"_1";
                else if ($scope.perc < 100) $scope.motivation = "DASH_MOT_"+$filter('uppercase')(cat.type)+"_2";
                else if ($scope.perc >= 100) $scope.motivation = "DASH_MOT_"+$filter('uppercase')(cat.type)+"_3";
        }


        $scope.perc = 0;
        $scope.total = 0;
        $scope.motivation = "";
        $scope.setMotivation($scope.active);

        $scope.refreshTimeout;
        $scope.refreshType;
        $scope.$on('newData', function(event, args) {
            if (!$scope.refreshType) $scope.refreshType = args.type;
            if (args.type == $scope.refreshType) {
                $scope.indicator = {
                    show:true,
                    text:$translate.instant('Aktualisiere Daten')
                }
                $timeout.cancel($scope.refreshTimeout);
                $scope.refreshTimeout = $timeout(function(){
                    initValues(true);
                    $scope.active = angular.copy($scope.categories[$scope.active.index]);
                    $scope.indicator.show = false;
                },1500);
            } else {
                $scope.refreshType = args.type;
                initValues(true);
                $scope.active = angular.copy($scope.categories[$scope.active.index]);
            }
        });

        $scope.getLinkUrl = function() {
            return $state.href('app.'+$scope.active.type,{categoryId:$scope.active.type});
        }

        /*
         *  GOAL
         */

        $scope.progVal = ($scope.active.fakeCurrent || $scope.active.current)/($scope.active.fakeGoal || $scope.active.goal)*100;

        $scope.goal = {
            "title":"",
            "text":"",
            "OK":"",
            "cancel":""
        }
        $scope.daytimeSince = 1;
        $translate(['Tagesziel', 'OK','Abbrechen']).then(function (translations) {
            $scope.goal.title = translations.Tagesziel;
            $scope.goal.OK = translations.OK;
            $scope.goal.cancel = translations.abbrechen;
        });

        $scope.dailyGoalChanged = function() {
            $translate(goals.message($scope.active.type)).then(function(translation) {
                var confirm = $mdDialog.prompt()
                  .title($scope.goal.title)
                  .textContent(translation)
                  .placeholder($scope.active.fakeGoal || $scope.active.goal)
                  .ariaLabel($scope.goal.title)
                  .initialValue($scope.active.fakeGoal || $scope.active.goal)
                  .ok($scope.goal.OK)
                  .cancel($scope.goal.cancel);
                $mdDialog.show(confirm).then(function(result) {
                    goals.set($scope.active.type, result);
                    if ($scope.active.type != "heart") {
                        $scope.categories[$scope.active.index]["goal"] = result;
                    } else {
                        $scope.categories[1].fakeGoal = result;
                        $scope.categories[1]['current'] = $scope.categories[1].goal - Math.abs( $scope.categories[1].fakeGoal - $scope.categories[1].fakeCurrent);
                    }
                    $scope.active = angular.copy($scope.categories[$scope.active.index]);
                });
            });
        }

        /*
         *  BLUETOOTH INDICATOR
         */

        $scope.btIndicatorState = 2;
        $scope.$on("btConnection", function(event, args) {
            $scope.btIndicatorState = args.state;
        });

        /*
         *  BUSINESS INDICATOR
         */

        $scope.openDetails = function(){
            $scope.indicator = {
                show:true,
                text:$translate.instant('Lade Daten')
            }
        };
        $scope.$on("$ionicView.afterLeave", function(event, data){
            $scope.indicator.show = false;
        });
        $scope.indicator = {
            show:false,
            text:$translate.instant('Aktualisiere Daten')
        }
    });
    
})
.controller('ImpressumController', function($scope, $ionicPlatform, $cordovaAppVersion){
    $scope.count = 0;
    $scope.showBuild = false;
    $scope.info = "";
    
    $scope.addCount = function() {
        $scope.count++;
        if ($scope.count >= 10) {
            $scope.showBuild = true;
        }
    }
    
    $ionicPlatform.ready(function () {
        $cordovaAppVersion.getVersionNumber().then(function (version) {
            var appVersion = version;
            $cordovaAppVersion.getVersionCode().then(function (build) {
                var appBuild = build;
                $scope.info = "v."+appVersion+" ("+appBuild+")";
            });
        });
    });

//    $cordovaAppVersion.getAppName().then(function (name) {
//        var appName = name;
//    });
//
//
//    $cordovaAppVersion.getPackageName().then(function (package) {
//        var appPackage = package;
//    });
});