angular.module('starter.services', ['btcontroller_interface'])
.factory("goals", function (localStorageService){
    var defaultGoals = {
        steps:8000,
        heart:100,
        calories:160,
        sleep:8
    }
    var goals = {
        steps:getGoal('steps'),
        heart:getGoal('heart'),
        calories:getGoal('calories'),
        sleep:getGoal('sleep')
    }
    var messages = {
        steps:"TEXT Tagesziel Schritte",
        heart:"TEXT Tagesziel Puls",
        calories:"TEXT Tagesziel Kalorien",
        sleep:"TEXT Tagesziel Schlaf"
    }
    
    function getGoal(type) {
        if (tmp = localStorageService.get("goals")) {
            if (tmp[type])
                return tmp[type];
        } 
        return defaultGoals[type];
    }
    
    return {
        refresh:function() {
            angular.forEach (['steps','heart','calories','sleep'], function (value){
                goals[value] = parseInt(getGoal(value));
            });
            return goals;
        },
        all: function () {
            return goals;
        },
        type: function (type) {
            return parseInt(goals[type]);  
        },
        set:function(type, value) {
            goals[type] = value;
            localStorageService.set("goals",goals);
        },
        message:function(type) {
            return messages[type];
        }
    }
    
})
.factory('dashboard', function(localStorageService, goals, $filter, btinterface) {  
    var today, hourData, heartData, sleepData;
    var hourValue, heartValue, sleepValue;
    
    // steps/calories
    var hour_getValues = function(moving_data, today) {
        var result = {
            s:0,
            d:0,
            c:0,
            ex_t:0,
            ex_a:0
        };
        if ((tmp = moving_data[today])) {
            var idx = tmp.length-1;
            result.s = tmp[idx].s;
            result.d = tmp[idx].d;
            result.c = tmp[idx].c;
            result.ex_t = tmp[idx].ex_t;
            result.ex_a = tmp[idx].ex_a;
        } 
        return result;
    }

    // heart rate
    var heart_getValues = function(h, _h) {
        if (h.day) {
            if (h.day.amountEntries > 0) return h.day.atRest.val;
        } 
        
        // no day/night data
        if (h.all) {
            if (h.all.amountEntries > 0) return h.all.atRest.val;
        }
        
        // no data
        return _h.day.atRest.val;  
    }
    
    var init = function() {
        today = moment().format('YYYY-MM-DD');
        
        heart_data = btinterface.byTagAndDate('heart', today);
        _heart_data = btinterface.heartRateAtRest(today);
        moving_data = localStorageService.get('moving_data') || {};
        
        hourValue = hour_getValues(moving_data, today);
        heartValue = heart_getValues(heart_data, _heart_data);
    }
    init();
    
    function sleepData() {
        return btinterface.byTagAndDate('sleep', moment()).then(
            function(result) {
                if (result)
                    if (result[0] && result[0]['utc_getup']) {
                        // choose first result
                        // TODO: change the way for deciding what was the actual sleep
                        var tmp = result[0];
                        return ((tmp.utc_getup-tmp.utc_sleep)/60 - tmp.m_awake);
                    } else return (0);
                else return (0);
            }
        )
    }
    
    function async(tag) {
        switch (tag) {
            case 'sleep':
                return sleepData();
                break;
        }
    }
    
    return {
        values: function() {
            return {
                's':hourValue.s || 0,
                'c':hourValue.c || 0,
                'd':hourValue.d || 0,
                'h':heartValue || 0
            }
        },
        refresh: init,
        async:async
    }
   
})

/*  Returns statistics over all entries, like total and highest values. 
 *  WARNING: all() and recursiveStats() return stats in a different object structure
 *  -> this should be the same (preferably recursiveStats' structure)
 */
.factory('stats', function(localStorageService){

    var recursiveStats = function (type) {
        switch (type) {
            case 'steps': 
            case 'calories': 
            case 'distance':
                var highest = {
                    'value':0,
                    'date':new Date().toJSON().split("T")[0]
                };
                var total = 0;
                var moving_data = localStorageService.get('moving_data') || {};
                for (key in moving_data) {
                    if (moving_data.hasOwnProperty(key)) {
                        var tmp = moving_data[key];
                        var last_entry = tmp[tmp.length-1];
                        var value = last_entry[type[0]];
                        if (value >= highest.value) {
                            highest.value = value;
                            highest.date = key
                        }
                        total += value;
                    }
                }
                return {
                    total:total,
                    highest:highest
                }
                break;
            default:
                return 0;
        }
    }
    
    var stats = {
        steps:getStat('steps'),
        heart:getStat('heart'),
        calories:getStat('calories'),
        distance:getStat('distance'),
        sleep:getStat('sleep')
    }
    
    function getStat(type) {
        tmp = localStorageService.get("stats") || {};
        if (tmp[type]) return tmp[type];
        else {
            var val = recursiveStats(type);
            tmp[type] = {
                date: new Date(),
                data: val
            }
            if (val != 0) localStorageService.set ("stats", tmp);
            
            return tmp;
        }
    }
    
    
    
    return {
        refresh:function(type) {
            if (!type) {
                angular.forEach (['steps','heart','distance','calories','sleep'], function (value){
                    stats[value] = getStat(value);
                });
            } else {
                stats[type] = getStat(type);
            }
            return stats;
        },
        recursive:function(type) {
             if (!type) {
                angular.forEach (['steps','heart','distance','calories','sleep'], function (value){
                    stats[value] = recursiveStats(value);
                });
            } else {
                stats[type] = recursiveStats(type);
            }
            return stats;
        },
        all: function () {
            return stats;
        },
        type: function (type) {
            return stats[type];  
        }
    }
})
.filter('numberspread', function() {
    
  return function(input) {
    input = input || 0;
    var tmp = input.toString();
    
    var output = "";
    if (tmp.indexOf(".") >= 0) {
        var arr = tmp.split(".");
        if (arr.length != 2) return input;
        output = "."+arr[1];
        tmp = arr[0];
    }
    
    while (tmp.length > 0) {
        if (tmp.length < 3) output = tmp + output;
        else output = " "+tmp.substring(tmp.length - 3)+output;
        tmp = tmp.substring(0, tmp.length - 3);
    }

    return output;

  }
})
.filter('numbertotime', function(){
    return function(input) {
        if (input == 0) return "0:00";
        var hh = Math.floor(input);
        var mm = pad(Math.floor((input%1) * 60));
        return hh+":"+mm;
    }
})
.filter('m2km', function(){
    return function(input) {
        if (input >= 1000) {
            return (input/1000).toFixed(1);
        } else {
            return input;
        }
    }
})
.filter('titlize', function(){
    return function(input) {
        if (typeof(input) === 'string') {
            var result = "";
            var arr = input.split(" ");
            for(var i=0;i<arr.length;i++) {
                var str = arr[i];
                if (str) {
                    if (i > 0) result += " ";
                    result += str[0].toUpperCase() + str.substr(1); 
                }
            };
            return result;
        }
        return input;
    }
})
.filter('moment', function(){
    return function(input, f) {
        var tmp = moment(input);
        if (tmp.isValid()) return tmp.format(f);
        else return tmp;
    }
})
.factory('motivational', function($translate){
    var mot_genr = {
        steps:4,
        calories:4,
        heart:4,
        sleep:4
    }
    var mot_perc = {
        steps:[{
                count:0,
                perc:0.3      
            },{
                count:5,    // how many texts are there
                perc:0.5    // max percentage of the goal to trigger these 
            },{
                count:5,
                perc:0.9
            },{
                count:5,
                perc:1
            },{
                count:5
        }],
        calories:[{
                count:0,
                perc:0.3     
            },{
                count:4,    // how many texts are there
                perc:0.5    // max percentage of the goal to trigger these 
            },{
                count:4,
                perc:0.9
            },{
                count:4,
                perc:1
            },{
                count:4
        }],
        heart:[{
                count:0,
                perc:0.2      
            },{
                count:4,
                perc:0.9
            },{
                count:4
        }],
        sleep:[{
                count:0,
                perc:0.5      
            },{
                count:4,
                perc:1
            },{
                count:4
        }]
    }
    
    var current = {};
    var getIdx = function(type, perc){
        var data = mot_perc[type];
        if (data.length == 1) return 0;         // no levelled values 
        var idx = 0;
        for (var i=0; i<data.length; i++) {
            if (data[i].perc && data[i].perc > perc) break;
            else if (i == data.length-1) {
                return i;                       // reached last level
            }
        }
        return idx;                             // some level in between
    };
    var getCount = function(type, idx){
        return mot_perc[type][idx]['count']; 
    };

    var mot = {
        getMot:function(type, perc){
            if (isNaN(perc)) perc = 0;  // fallback for NaN values
            var idx = getIdx(type,perc);
            if (isNaN(current[type]) || idx != current[type]) {
                // either first or crossed a level
                current[type] = idx;
                var count_perc = getCount(type,idx);   
                var count_genr = mot_genr[type];   
                if (perc >= mot_perc[type][mot_perc[type].length-1]['perc']) count_genr = 0;
                var count = count_perc+count_genr;     
                var num = Math.floor(Math.random() * count); 
                var text = "";
                if (num < count_perc) 
                    text =  "TEXT MOT "+type+"_"+idx+"_"+num;
                else 
                    text = "TEXT MOT ALLG "+type+"_"+(num-count_perc);
                console.log("MOT TEXT: "+text);
                return text;
            } else {
                return false;
            }
        }
    }
    return mot;
});