angular.module('sleep', ['btcontroller_interface']) 

.controller('SleepController', function($scope, localStorageService, $filter, $timeout, goals, bt_parser, $q, $ionicScrollDelegate, btinterface, alt_sleepDayOptions, alt_weeksSleep, alt_monthsSleep, $translate, $filter) {
    
    // when today is displayed and new data comes in
    $scope.$on('newData', function(event, args){
        if (args.type == 'sleep') {
            if ($scope.selected_date == moment().subtract(1,'d').format("YYYY-MM-DD")) {
                alt_weeksSleep.refresh();
                alt_monthsSleep.refresh();
                
                $scope.hour_change(0);
            }
        };
    });
    
    // get initial date
    $scope.selected_date = moment().format('YYYY-MM-DD');
    
    // chart settings
    $scope.sleep_options = alt_sleepDayOptions.options();;
    $scope.datasetOverride = alt_sleepDayOptions.datasetOverride();
    $scope.colors = ['#45b7cd'];//['#45b7cd', '#ff6384', '#ff8e72'];
    $scope.series = ['bar-data'];
    $scope.parsedData = localStorageService.get("sleep_data_overview") || [];
    
    $scope.goal = goals.type('sleep');
    
    $scope.hour_change = function (add) {
    
        // reset
        $scope.totalSleep = 0;
        
        // get target date
        $scope.selected_date = moment($scope.selected_date).add(add,'d').format('YYYY-MM-DD');
        $scope.displayDate = moment($scope.selected_date).format('dd, DD.MM.YY');
        
        btinterface.byTagAndDate('sleep',$scope.selected_date).then(function(success){
            
            
            // reset everything
            $scope.sleep_data = [[],[],[]];
            $scope.sleep_labels = [];
            $scope.totals =  [
                {perc:0,date:'00:00'},
                {perc:0,date:'00:00'},
                {perc:0,date:'00:00'}
            ]
            $scope.totalSleep = 0;
            $scope.totalSleepTime = null;
            $scope.totalSleepPercentage = 0;
            
            if (success) {
                // combine data
                var allTotals = [];
                var splitData = success.length > 1 ? [[],[],[],[],[],[]] : [[],[],[]];
                var labels = [];
                var count = 0;
                success.forEach(function(data) {
                    if (Object.keys(data).length > 0) {
                        count++;
                        var d_ref;
                        var data_range_valid = true;
                        if (count > 1) {
                            d_ref = labels[labels.length-1];
                            // set the first label-value to 0 so there is a straight edge
                            // since date can be before last added label due to seperate native algorithm queries
                            // get the first date that is coming after last label
                            var ddl = data.data.length;
                            var ref_count = 0;
                            var date = moment(data.data[ref_count].utc_s*1000);
                            while (date.isBefore(d_ref)) {
                                date = moment(data.data[ref_count].utc_s*1000);
                                ref_count++;
                                if (ref_count >= ddl) break; 
                            }
                            if (date.isSameOrAfter(d_ref)) {
                                labels.push(date);
                                [0,1,2,3,4,5].forEach(function(i) {
                                    splitData[i].push(0);
                                });
                            } else data_range_valid = false;
                        }
                        if (!data_range_valid) return; // continue and skip entire entry with its data
                        data.data.forEach(function(val) {
                            var d = moment(val.utc_s*1000);
                            if (d_ref && d.isBefore(d_ref)) return; // continue and skip this data.data entry
                            labels.push(d);
                            labels.push(moment((val.utc_s+val.m_d*60)*1000));
                            switch (val.status) {
                                case 1:
                                    var _ibase, base;
                                    if (success.length > 1) {
                                        _ibase = count == 1 ? [0, [1,2,3,4,5]] : [3, [0,1,2,4,5]];
                                        ibase = _ibase[0];
                                    } else {
                                        _ibase = [0, [1,2]];
                                        ibase = _ibase[0];
                                    }
                                    splitData[ibase].push(100);
                                    splitData[ibase].push(100);
                                    break;
                                case 2:
                                    var _ibase, base;
                                    if (success.length > 1) {
                                        _ibase = count == 1 ? [1, [0,2,3,4,5]] : [4, [0,1,2,3,5]];
                                        ibase = _ibase[0];
                                    } else {
                                        _ibase = [1, [0,2]];
                                        ibase = _ibase[0];
                                    }
                                    splitData[ibase].push(66);
                                    splitData[ibase].push(66);
                                    break;
                                case 3:
                                    var _ibase, base;
                                    if (success.length > 1) {
                                        _ibase = count == 1 ? [2, [0,1,3,4,5]] : [5, [0,1,2,3,4]];
                                        ibase = _ibase[0];
                                    } else {
                                        _ibase = [2, [0,1]];
                                        ibase = _ibase[0];
                                    }
                                    splitData[ibase].push(30);
                                    splitData[ibase].push(30);
                                    break;
                            }
                            _ibase[1].forEach(function(i) {
                                splitData[i].push(0);
                                splitData[i].push(0);
                            });
                        });
                    }
                    if (count < success.length) {
                        labels.push(moment(labels[labels.length-1]));
                        [0,1,2,3,4,5].forEach(function(i) {
                            splitData[i].push(0);
                        });
                    }
                    
                    var total = data.m_deep + data.m_light + data.m_awake;
                    var totals =  [
                        {perc:data.m_deep/total,date:pad(Math.floor(data.m_deep/60))+":"+pad(data.m_deep%60)},
                        {perc:data.m_light/total,date:pad(Math.floor(data.m_light/60))+":"+pad(data.m_light%60)},
                        {perc:data.m_awake/total,date:pad(Math.floor(data.m_awake/60))+":"+pad(data.m_awake%60)}
                    ]
                    var totalSleep = (total-data.m_awake) / 60;
                    var totalSleepTime = pad(Math.floor(totalSleep))+":"+pad(Math.floor((total-data.m_awake)%60));
                    allTotals.push({
                        totals:totals,
                        totalSleep:totalSleep,
                        totalSleepTime:totalSleepTime
                    });
                });


                $scope.datasetOverride = [
                    {
                        label: $filter('titlize')($translate.instant("Aktive Phasen")),
                        type: 'line',
                        backgroundColor : '#eada96',
                        borderColor: '#eada96'
                    },{
                        label: $filter('titlize')($translate.instant("Ruhephasen")),
                        type: 'line',
                        backgroundColor : '#93dced',
                        borderColor: '#93dced'
                    },{
                        label: $filter('titlize')($translate.instant("Tiefschlaf")),
                        type: 'line',
                        backgroundColor : '#807cd2',
                        borderColor: '#807cd2'
                    },{
                        label: $filter('titlize')($translate.instant("Aktive Phasen")),
                        type: 'line',
                        backgroundColor : '#f9f5e4',
                        borderColor: '#f9f5e4'
                    },{
                        label: $filter('titlize')($translate.instant("Ruhephasen")),
                        type: 'line',
                        backgroundColor : '#e3f6fa',
                        borderColor: '#e3f6fa'
                    },{
                        label: $filter('titlize')($translate.instant("Tiefschlaf")),
                        type: 'line',
                        backgroundColor : '#dfdef3',
                        borderColor: '#dfdef3'
                    }
                    
                ];
                $scope.sleep_series = ['awake','light','deep','awake','light','deep'];
                $scope.sleep_data = splitData;
                $scope.sleep_labels = labels;
                
                first = allTotals[0];
                $scope.totals = first.totals;
                $scope.totalSleep = first.totalSleep;
                $scope.totalSleepTime = first.totalSleepTime;
                $scope.totalSleepPercentage = Math.floor($scope.totalSleep / $scope.goal * 100);
                
                // parsed data (for list)
                p_found = false;
                for (var k = 0, len = $scope.parsedData.length; k < len; k++) {
                    if ($scope.parsedData[k]['d'] == $scope.selected_date) p_found = true;
                }
                if (!p_found) {
                    $scope.parsedData.push({
                        t:$scope.totals,
                        st:$scope.totalSleepTime,
                        d:$scope.selected_date
                    });
                    $scope.parsedData.sort(function(a,b){
                        if (a.d < b.d) return 1;
                        if (a.d > b.d) return -1;
                        return 0;
                    });
                    localStorageService.set('sleep_data_overview', $scope.parsedData);
                }
            }
      }, function(error){
            console.log("sFINAL: "+JSON.stringify(error));
      });
    }
    
    // get initial sleep values (current day/night)
    $scope.hour_change(0);
    
    /*
     *  VERLAUF
     */
    
    $scope.jumpTo = function (value) {
        $scope.bar_setActive(0);
        var target = moment(value.d);
        var active = moment($scope.selected_date);
        var diff = target.diff(active, 'days');
        $scope.hour_change(diff);
        $ionicScrollDelegate.scrollTop(true);
    }
    
    /*
     *  switch between day, week and month
     */
    
    $scope.bar_active = 0;
    $scope.bar_setActive = function(type) {
        $scope.bar_active = type;
        $scope.switchDataType (type); 
    };
    $scope.bar_isActive = function(type) {
        return type === $scope.bar_active;
    };
    
        
    $scope.activeOverview;
    $scope.overview = {}
 
    
    $scope.switchDataType = function(type) {
        switch (type) {
            case 0:   
                break;
            case 1:
                $scope.activeOverview = alt_weeksSleep;
                switchToOverview();
                break;
            case 2:
                $scope.activeOverview = alt_monthsSleep;
                switchToOverview();
                break;
        }
    }
    
    $scope.controls = function(add) {
        switch ($scope.bar_active) {
            case 0:
                $scope.hour_change(add);
                break;
            case 1:
            case 2:
                $scope.activeOverview.change(add);
                switchToOverview();
                break;
        }
    }
    
    /*
     *  OVERVIEW (WEEK/MONTH)
     */

    function switchToOverview () {
        
        // is new data required
        if ($scope.activeOverview.current() != (($scope.bar_active == 1) ? $scope.overview.week : $scope.overview.month)) {
            // get new data
            $scope.activeOverview.data().then(function(data) {
                $scope.overview = data.overview;
                 
                // set title
                if ($scope.overview.options.title.text != $scope.activeOverview.title())
                    $scope.overview.options.title.text = $scope.activeOverview.title();
                    
                // percentage
                if ($scope.overview.average && $scope.overview.average != "0:00") {
                    var tmpArr = $scope.overview.average.split(":");
                    var tmpNum = parseInt(tmpArr[0]) + (parseInt(tmpArr[1]) / 60);

                    $scope.overviewPerc = Math.floor(tmpNum / (goals.type('sleep') * $scope.overview.count) * 100);
                    $scope.overviewCurrent = tmpNum;
                } else {
                    $scope.overviewPerc = 0;
                }
            });
        } 
    }
    
})

.factory('alt_weeksSleep', function($filter, $q, localStorageService, sleepOverview, btinterface, $translate, $filter){
    // TODO: store last data
    var currentWeek = -1;
    var options = {};
    var currentYear = moment().year();
    
    return {
        refresh: function(){
            sleep_data = localStorageService.get('sleep_data') || {};
            sleep_data_parsed = localStorageService.get('sleep_data_parsed') || {};
        },
        title: function(){
            return $filter('titlize')($translate.instant("Wochenübersicht"));
        },
        current: function(){
            return currentWeek;
        },
        currentYear: function() {
            return currentYear;  
        },
        change: function(add) {
            var tmpD = moment().year(currentYear).month(0).week(currentWeek + add);
            currentWeek = tmpD.week();
            currentYear = tmpD.year();
        },
        data: function(){
            // fetch options if needed
            if (Object.keys(options).length == 0) 
                options = sleepOverview.options();

            // get currentWeek
            if (currentWeek < 0) {
                currentWeek = moment().week();
            }

            // fetchData and create labels
            var labels = [];
            var sleep = [];
            var total = 0; // total sleep in hours
            var totalRecorded = 0; // total milliseconds recorded with values
            var totals = [0,0,0];
            var _totals = [{},{},{}];
            var count = 0; // in days
            var tmpD = moment().month(0).year(currentYear).week(currentWeek).startOf('week'); 
            var indexes = [];
            var date = moment().year(currentYear).month(0).week(currentWeek).format('w')+". "+$translate.instant('_Kalenderwoche');
            
            // decide if next button should be blocked
            var limit = false;
            if (currentWeek == moment().week() && moment().year() == currentYear) {
               limit = true;
            } 
            
            // get arrays of labels and indexes
            while (tmpD.week() == currentWeek) {
                labels.push(angular.copy(tmpD));
                sleep.push(0);
                indexes.push(tmpD.format('YYYY-MM-DD'));
                tmpD.add(1, 'd'); 
            }
            
            // get array of promises -> sleepData of each day
            var deferred = $q.defer(); 
            var promises = [];
            for (var i = 0; i < indexes.length; i++) {
                promises.push(btinterface.byTagAndDate('sleep',indexes[i]).then(function(success){
                    if (success) {
                        count++;
                        var n_data = success[0];  // TODO: ASSUMES THAT THIS IS THE ACTUAL SLEEP
                        var n_diff_h = (n_data.utc_getup - n_data.utc_sleep)/3600;
                        
                        var idx = parseInt(moment(n_data.date).format('d'))-1;
                        sleep[idx] = n_diff_h;
                        total += n_diff_h;

                        totals[0] += (n_data.m_deep/60);
                        totals[1] += (n_data.m_light/60);
                        totals[2] += (n_data.m_awake/60);
                    } 
                }))
            }
            
            // once all promises have return their values
            $q.all(promises).then(function(){
                // total
                var average_time = "0:00";
                if (count > 0 && total > 0) {
                    var average = Math.floor(total / count * 60); // minutes
                    average_time = pad(Math.floor(average/60))+":"+pad(average%60);
                } 


                // totals
                _totals = [
                    {perc:totals[0]/total,date:pad(Math.floor(totals[0]))+":"+pad(Math.floor(totals[0]*60)%60)},
                    {perc:totals[1]/total,date:pad(Math.floor(totals[1]))+":"+pad(Math.floor(totals[1]*60)%60)},
                    {perc:totals[2]/total,date:pad(Math.floor(totals[2]))+":"+pad(Math.floor(totals[2]*60)%60)}
                ];

                deferred.resolve({'overview':{
                    sleep:[sleep],
                    labels:labels,
                    date:date,
                    count:count,
                    total:total,
                    limit:limit,
                    options:options,
                    average:average_time,
                    totals:_totals,
                    series:[$filter('titlize')($translate.instant("Schlaf"))],
                    colors:['#807cd2']
                }});
            })
            
            return deferred.promise;
        }
    }
})

.factory('alt_monthsSleep', function($filter, $q, localStorageService, sleepOverview, btinterface, $translate, $filter){
    // TODO: store last data
    var currentMonth = -1;
    var sleep_data, sleep_data_parsed;
    var options = {};
    var currentYear = moment().year();
    
    return {
        refresh: function(){
            sleep_data = localStorageService.get('sleep_data') || {};
            sleep_data_parsed = localStorageService.get('sleep_data_parsed') || {};
        },
        title: function(){
            return $filter('titlize')($translate.instant("Monatsübersicht"));
        },
        current: function(){
            return currentMonth;
        },
        currentYear: function() {
            return currentYear;  
        },
        change: function(add) {
            var tmpD = moment().year(currentYear).month(currentMonth + add);
            currentMonth = tmpD.month();
            currentYear = tmpD.year();
        },
        data: function(){
            // fetch options if needed
            if (Object.keys(options).length == 0) 
                options = sleepOverview.options();

            // get currentMonth
            if (currentMonth < 0) {
                currentMonth = moment().month();
            }

            if (!sleep_data) sleep_data = localStorageService.get('sleep_data') || {};
            if (!sleep_data_parsed) sleep_data_parsed = localStorageService.get('sleep_data_parsed') || {};

            // fetchData and create labels
            var labels = [];
            var sleep = [];
            var total = 0; // total sleep in hours
            var totalRecorded = 0; // total milliseconds recorded with values
            var totals = [0,0,0];
            var _totals = [{},{},{}];
            var count = 0; // in days
            var indexes = [];
            var date = moment().month(currentMonth).year(currentYear).format('MM/YY');
            
            // decide if next button should be blocked
            var limit = false;
            if (currentMonth == moment().month() && moment().year() == currentYear) {
               limit = true;
            } 
            
            // get arrays of labels and indexes
            var tmpD = moment().month(currentMonth).year(currentYear).startOf('month'); 
            while (tmpD.month() == currentMonth) {
                indexes.push(tmpD.format('YYYY-MM-DD'));
                labels.push(angular.copy(tmpD));
                sleep.push(0);
                tmpD.add(1, 'd'); 
            }
            
            // get array of promises -> sleepData of each day
            var deferred = $q.defer(); 
            var promises = [];
            for (var i = 0; i < indexes.length; i++) {
                promises.push(btinterface.byTagAndDate('sleep',indexes[i]).then(function(success){
                    if (success) {
                        count++;
                        var n_data = success[0];  // TODO: ASSUMES THAT THIS IS THE ACTUAL SLEEP
                        var n_diff_h = (n_data.utc_getup - n_data.utc_sleep)/3600;

                        var idx = parseInt(moment(n_data.date).format('D'))-1;
                        sleep[idx] = n_diff_h;
                        total += n_diff_h;

                        totals[0] += (n_data.m_deep/60);
                        totals[1] += (n_data.m_light/60);
                        totals[2] += (n_data.m_awake/60);
                    }
                }))
            }
            
            // once all promises have return their values
            $q.all(promises).then(function(){
                // total
                var average_time = "0:00";
                if (count > 0 && total > 0) {
                    var average = Math.floor(total / count * 60); // minutes
                    average_time = pad(Math.floor(average/60))+":"+pad(average%60);
                } 

                // totals
                _totals = [
                    {perc:totals[0]/total,date:pad(Math.floor(totals[0]))+":"+pad(Math.floor(totals[0]*60)%60)},
                    {perc:totals[1]/total,date:pad(Math.floor(totals[1]))+":"+pad(Math.floor(totals[1]*60)%60)},
                    {perc:totals[2]/total,date:pad(Math.floor(totals[2]))+":"+pad(Math.floor(totals[2]*60)%60)}
                ];

                deferred.resolve({'overview':{
                    sleep:[sleep],
                    labels:labels,
                    date:date,
                    count:count,
                    total:total,
                    limit:limit,
                    options:options,
                    average:average_time,
                    totals:_totals,
                    series:[$filter('titlize')($translate.instant("Schlaf"))],
                    colors:['#807cd2']
                }});
            })
            
            return deferred.promise;
        }
    }
})

.factory('sleepOverview', function($translate, $filter){
    return {
        options: function(){
            return {
                scales: {
                    yAxes: [
                        {
                            id: 'y-axis-1',
                            type: 'linear',
                            position: 'left',
                            ticks:{
                                display:true,
                                beginAtZero:true,
                                steps:10,
                                fontSize: 8
                            },
                            gridLines: {
                                display:false
                            }
                        }
                    ],
                    xAxes: [{
                        barPercentage:0.5,
                        categoryPercentage:0.5,
                        type: 'time',
                        ticks:{
                            fontSize: 8,
                            fontColor: '#878787'
                        },
                        time: {
                            unit:'day',
                            unitStepSize: 1,
                            displayFormats: {
                                'day': 'DD.'
                            }
                        }
                    }]
                },
                tooltips: {
                    enabled:true,
                    callbacks:{
                        title: function(tooltipItem, data) {
                            return $filter('titlize')($translate.instant("Datum"))+": "+tooltipItem[0].xLabel.format('L');
                        }
                    }
                },
                legend: {
                    display:false
                },
                elements: {
                    line: {
                        tension: 0.1,                              
                        backgroundColor:'rgba(255,0,255,1)',      
                        borderColor:'rgba(255,0,255,1)',           
                        fill:false                                 
                    },
                    point: {
                        radius:0
                    }
                },
                defaultFontColor:'#878787',
                responsive:true,                                  
                title: {
                    fontColor:'#878787',
                    fontSize:14,
                    display: true
                }
            };
        }
    }
})

.factory('alt_sleepDayOptions', function($translate, $filter){
    return {
        datasetOverride: function() {
            return [{
                backgroundColor:[],
                borderColor:[],
                borderWidth:0
            }];
        },
        options: function(){
            return {
                scales: {
                    yAxes: [
                        {
                            id: 'y-axis-1',
                            type: 'linear',
                            position: 'left',
                            ticks:{
                                display:false,
                                beginAtZero:true,
                                steps:10,
                                fontSize: 8
                            },
                            gridLines: {
                                display:false
                            }
                        }
                    ],
                    xAxes: [{
                        type: 'time',
                        ticks:{
                            fontSize: 8
                        },
                        unit:'minute',
                        unitStepSize: 5,
                        gridLines: {
                            display:false
                        },
                        time: {
                            format: "HH",
                            unit: 'hour',
                            //unitStepSize: 3,
                            displayFormats: {
                                'day': 'HH',
                                'hour': 'HH'
                            }
                        }
                    }]
                },
                tooltips: {
                    enabled:true,
                    callbacks:{
                        title: function(tooltipItem, data) {
                            return $filter('titlize')($translate.instant("Datum"))+": "+tooltipItem[0].xLabel.format('L');
                        }
                    }
                },
                legend: {
                    display:false
                },
                elements: {
                    line: {
                        tension: 0,                         
                        fill:true                                 
                    },
                    point: {
                        radius:0
                    }
                },
                defaultFontColor:'#878787',
                responsive:true,                                  
                title: {
                    fontColor:'#878787',
                    fontSize:14,
                    display: true
                },
                zoom: {
                    enabled: true,
                    mode: 'x',
                },
                pan: {
                    enabled: true,
                    mode: 'x',
                }
            };
        }
    }
});