angular.module('steps', ['btmessage', 'btconnect'])

.controller('StepsController', function($scope, localStorageService, stats, $timeout, goals, $ionicScrollDelegate, weekOverview, monthOverview, generalOverview, yearOverview, dayOverview, dayOverview_options, bt_sms_messenger,mambo2_address, $translate, $filter) {
  
    // vars
    $scope.active = {};
    $scope.stats = {};
    $scope.activeOverview;
    $scope.overview = {
        options: {},
        data: [],
        labels: [],
        date:moment().format('dd, DD.MM.YY'),
        limit: false
    }
    
    var setStats = function () {
        stats.refresh();
        $scope.stats = stats.recursive(); 
    };
    setStats();   
    
    // (re-)initialize values on entering
    $scope.$on('$ionicView.enter', function() {
        $scope.controls(0);
        daytimes();
        
        // TEST
        // bt_sms_messenger.send_title({'name':'Tobias','number':'017623680939'}, mambo2_address.get(), "Hallo");
    });
    
    // when today is displayed and new data comes in
    $scope.$on('newData', function(event, args){
        if (args.type == 'moving') {
            if ($scope.hour_selected == moment().format("YYYY-MM-DD")) {
                dayOverview.refresh();
                weekOverview.refresh();
                monthOverview.refresh();
                yearOverview.refresh();
                
                $scope.bar_setActive(0);
                $scope.controls(days);
            }
            daytimes();
        };
    });
    
    // chart defaults
    $scope.hour_options = dayOverview_options.options();
    $scope.hour_colors = ['#89cc2e', '#e1e100'];
    $scope.hour_series = [$filter('titlize')($translate.instant('Schritte')),$filter('titlize')($translate.instant('Kalorien'))];
    $scope.hour_selected = moment().format('YYYY-MM-DD');
    $scope.hour_options.legend['onClick'] = function(e,p){
        var tmpBool = $scope.hour_datasetOverride[p.datasetIndex].hidden;
        $timeout(function(){
            $scope.hour_options.scales.yAxes[p.datasetIndex].ticks.display = tmpBool;
            $scope.hour_datasetOverride[p.datasetIndex].hidden = !tmpBool;
             if ($scope.week_options) {
                 $scope.week_options.scales.yAxes[p.datasetIndex].ticks.display = tmpBool;
             }
        });
    }
    $scope.hour_datasetOverride = [
        { 
            yAxisID: 'y-axis-1', 
            hidden:false, 
            fill:true,
            lineTension:0.1
        }, { 
            yAxisID: 'y-axis-2',
            hidden:true, 
            fill:true,
            lineTension:0.1
        }
    ];
    
    /*
     *  VERLAUF
     */
    function daytimes() {
        var tmp_daytimes = [];
        var moving_data = localStorageService.get('moving_data') || {};
        for (key in moving_data) {
            if (moving_data.hasOwnProperty(key)) {
                var tmp = moving_data[key];
                var _tmp = tmp[tmp.length-1];
                _tmp['date'] = key;
                
                if (_tmp.s != 0 || _tmp.d != 0 || _tmp.c != 0) tmp_daytimes.push(tmp[tmp.length-1]);
            }
        }
        $scope.daytimes = tmp_daytimes;
    }
    
    $scope.jumpTo = function (value) {
        $scope.bar_setActive(0);
        var target = moment(value.date);
        var active = moment($scope.hour_selected);
        var days = target.diff(active, 'days');
        $scope.controls(days);
        $ionicScrollDelegate.scrollTop(true);
    }
    
    /*
     *  switch between day, week, month and year
     */
    
    $scope.bar_active = 0;
    $scope.bar_setActive = function(type) {
        $scope.bar_active = type;
        $scope.switchDataType (type); 
    };
    $scope.bar_isActive = function(type) {
        return type === $scope.bar_active;
    };
    
    $scope.switchDataType = function(type) {
        switch (type) {
            case 0:   
                $scope.activeOverview = dayOverview;
                break;
            case 1:
                $scope.activeOverview = weekOverview;
                break;
            case 2:
                $scope.activeOverview = monthOverview;
                break;
            case 3:
                $scope.activeOverview = yearOverview;
                break;
        }
        switchToOverview();
    }
    $scope.bar_setActive(0);
    
    /*
     *  OVERVIEW 
     */
    
    $scope.controls = function(add) {
        $scope.activeOverview.change(add);
        switchToOverview();
    }
    
    function activeData (totalData, count, monthsData) {
        var oActive = totalData;
        if ($scope.bar_active == 0) {
            // day
            oActive['goal'] = goals.type('steps');
            oActive['steps_percentage'] = Math.floor(oActive.steps / oActive.goal * 100);
        } else {
            if ($scope.bar_active != 3) {
                // week/month
                oActive['goal'] = count * goals.type('steps');
            } else {
                // year
                oActive['goal'] = 0;
                for (var i = 0; i < monthsData.length; i++) {
                    var tmpGoal = monthsData[i] * goals.type('steps');
                    oActive.goal += tmpGoal;
                }
            }
            var perc;
            if (oActive.goal > 0) perc = Math.floor(oActive.steps / oActive.goal * 100);
            else {
                perc = 0;
                oActive.goal = goals.type('steps');
            }
            oActive['steps_percentage'] = perc;
        }
        return oActive;
    }
    
    function switchToOverview () {
        // fetch options if needed
        if (Object.keys($scope.overview.options).length == 0) 
            $scope.overview.options = generalOverview.options();
        
        // set title
        if ($scope.overview.options.title.text != $scope.activeOverview.title()) {
            $scope.overview.options.title.text = $scope.activeOverview.title();
            $scope.overview.options.scales.xAxes[0] = $scope.activeOverview.override();
        }
        
        // get new data
        $scope.activeOverview.data().then(function(data) {
            $scope.overview.data = data.data;
            $scope.overview.labels = data.labels;
            $scope.overview.date = data.date;
            $scope.overview.limit = data.limit;
            $scope.active = activeData(data.total, data.count, data.monthsData);
        });
    }
})

.factory('dayOverview_options', function($translate, $filter){
    var day_options = {
        scales: {
            yAxes: [
                {
                    id: 'y-axis-1',
                    type: 'linear',
                    position: 'left',
                    ticks:{
                        display:true,
                        beginAtZero:true,
                        steps:10,
                        fontSize: 8
                    },
                    gridLines: {
                        display:false
                    }
                },
                {
                    id: 'y-axis-2',
                    type: 'linear',
                    position: 'right',
                    ticks:{
                        display:false,
                        beginAtZero:true,
                        steps:10,
                        fontSize: 8
                    },
                    gridLines: {
                        display:false
                    }
                }
            ],
            xAxes: [{
                type: 'time',
                ticks:{
                    fontSize: 8
                },
                format: "HH:mm",
                unit: 'hour',
                unitStepSize: 2,
                time: {
                    format: "HH:mm",
                    unit: 'hour',
                    unitStepSize: 2,
                    displayFormats: {
                      'minute': 'HH', 
                      'hour': 'HH', 
                    }
                }
            }]
        },
        tooltips: {
            enabled:true,
            callbacks:{
                title: function(tooltipItem, data) {
                    return $translate.instant('Datum')+": "+tooltipItem[0].xLabel.format('DD.MM.YYYY');
                }
            }
        },
        legend: {
            display:true,
            fullWidth:true,
            labels:{
                boxWidth:40
            }
        },
        elements: {
            line: {
                tension: 0.1,                               // bezier 'curving' 0-straight 1-whohoo
                fill:false                                  // remove the filling entirely
            },
            point: {
                radius:0
            }
        },
        defaultFontColor:'#878787',
        responsive:true,                                   // fits it on the screen, yeah
        title: {
            fontColor:'#878787',
            fontSize:14,
            display: true,
            text: $filter('titlize')($translate.instant('Stundendaten'))
        },
        zoom: {
            enabled: true,
            mode: 'x',
        },
        pan: {
            enabled: true,
            mode: 'x',
        }
    };
    
    return {
        options:function(){
            return day_options;
        }
    }
})

.factory('yearOverview', function($q, localStorageService, $translate, $filter) {
    var moving_data;
    var currentYear = -1;
    
    return {
        refresh: function() {
            moving_data = localStorageService.get('moving_data') || {};
        },
        override: function() {
            return {
                type: 'time',
                ticks:{
                    fontSize: 8,
                    fontColor:'#878787'
                },
                time: {
                    unit:'month',
                    unitStepSize: 1,
                    displayFormats: {
                        'month': 'MMM'
                    }
                }
            }
        },
        title: function(){
            return $filter('titlize')($translate.instant("Jahresübersicht"));
        },
        current: function(){
            return currentYear;
        },
        change: function(add) {
            var tmpD = moment().year(currentYear + add);
            currentYear = tmpD.year();
        },
        data: function(getCalories){
            return $q(function(resolve) { 
                // get current -> needs to be done here because of check for same year
                if (currentYear < 0) {
                    currentYear = moment().year();
                }

                if (!moving_data) moving_data = localStorageService.get('moving_data') || {};

                // fetchData and create labels
                var steps = [];
                var calories = [];
                var distance = [];
                var labels = [];
                var mData = [];
                var ex_amount = 0;
                var ex_time = 0;
                var total = {
                    steps:0,
                    calories:0,
                    distance:0
                };
                var count = 0;
                var tmpD = moment().year(currentYear).startOf('year'); 
                while (tmpD.year() == currentYear) {
                    // push date to labels
                    labels.push(angular.copy(tmpD));
                    var refMonth = tmpD.month();
                    
                    var mCount = 0;
                    var mSteps = 0;
                    var mCalories = 0;
                    var mDistance = 0;
                    var mEx_amount = 0;
                    var mEx_time = 0;
                    
                    while (tmpD.month() == refMonth) {
                        // get week and check for date
                        var idx = tmpD.format('YYYY-MM-DD');
                        if ((day = moving_data[idx])) {
                            var d = day[day.length-1];

                            total.steps += d.s;
                            mSteps += d.s;
                            total.calories += d.c;
                            mCalories += d.c;
                            total.distance += d.d;
                            mDistance += d.d;
                            mEx_amount += d.ex_a;
                            mEx_time += d.ex_t;
                            mCount++;
                        } 
                        
                        // add one day
                        tmpD.add(1,'d')
                    }
                    
                    // now sum up data
                    mData.push(mCount);
                    steps.push(mSteps);
                    calories.push(mCalories);
                    distance.push(mDistance);
                    if (mSteps != 0 || mCalories != 0 || mDistance != 0) count++;
                    ex_amount += mEx_amount;
                    ex_time += mEx_time;
                }

                total.calories = Math.round(total.calories);
                total.distance = Math.round(total.distance/1000);
                var date = moment().year(currentYear).format('YYYY');
                
                // decide if next button should be blocked
                var limit = false;
                if (currentYear == moment().year()) {
                   limit = true;
                } 
                
                // data/additional_data
                var data,additional_data;
                if (!getCalories) {
                    data = steps;
                    additional_data = {
                        calories:calories
                    };
                } else {
                    data = calories;
                    additional_data = {
                        steps:steps
                    };
                } 
                additional_data['distance'] = distance;
                additional_data['ex_amount'] = ex_amount;
                additional_data['ex_time'] = ex_time;
                
                resolve ({
                    data:[data],
                    additional_data:additional_data,
                    labels:labels,
                    total:total,
                    count:count,
                    date:date,
                    monthsData:mData,
                    limit:limit
                });
            });
            
        }
    }
})

.factory('monthOverview', function($q, localStorageService, $translate, $filter) {
    var current = -1;
    var moving_data;
    var currentYear = moment().year();
    
    return {
        refresh: function() {
            moving_data = localStorageService.get('moving_data') || {};
        },
        override: function() {
            return {
                barPercentage:0.5,
                categoryPercentage:0.5,
                type: 'time',
                ticks:{
                    fontSize: 8,
                    fontColor:'#878787'
                },
                time: {
                    unit:'day',
                    unitStepSize: 1,
                    displayFormats: {
                        'day': 'DD.'
                    }
                }
            }
        },
        title: function(){
            return $filter('titlize')($translate.instant("Monatsübersicht"));
        },
        current: function(){
            return current;
        },
        currentYear: function() {
            return currentYear;  
        },
        change: function(add) {
            var tmpD = moment().year(currentYear).month(current + add);
            current = tmpD.month();
            currentYear = tmpD.year();
        },
        data: function(getCalories){
            return $q(function(resolve) { 
                // get current
                if (current < 0) {
                    current = moment().month();
                }

                if (!moving_data) moving_data = localStorageService.get('moving_data') || {};

                // fetchData and create labels
                var steps = [];
                var calories = [];
                var distance = [];
                var labels = [];
                var ex_amount = 0;
                var ex_time = 0;
                var total = {
                    steps:0,
                    calories:0,
                    distance:0
                };
                var count = 0;
                var tmpD = moment().year(currentYear).month(current).startOf('month'); 
                while (tmpD.month() == current) {
                    // push date to labels
                    labels.push(angular.copy(tmpD));

                    // get week and check for date
                    var idx = tmpD.format('YYYY-MM-DD');
                    if ((day = moving_data[idx])) {
                        var d = day[day.length-1];
                        
                        total.steps += d.s;
                        steps.push(d.s);
                        total.calories += d.c;
                        calories.push(d.c);
                        total.distance += d.d;
                        distance.push(d.d);
                        ex_amount += d.ex_a;
                        ex_time += d.ex_t;
                        count++;
                    } else {
                        // no data for this day
                        steps.push(0);
                        calories.push(0);
                        distance.push(0);
                    }

                    // add one day
                    tmpD.add(1, 'd'); 
                }
                
                total.calories = Math.round(total.calories);
                total.distance = Math.round(total.distance/1000);
                var date = moment().month(current).year(currentYear).format("MMMM 'YY");
                
                // decide if next button should be blocked
                var limit = false;
                if (current == moment().month() && moment().year() == currentYear) {
                   limit = true;
                } 
                
                // data/additional_data
                var data,additional_data;
                if (!getCalories) {
                    data = steps;
                    additional_data = {
                        calories:calories
                    };
                } else {
                    data = calories;
                    additional_data = {
                        steps:steps
                    };
                } 
                additional_data['distance'] = distance;
                additional_data['ex_amount'] = ex_amount;
                additional_data['ex_time'] = ex_time;
                
                resolve ({
                    data:[data],
                    additional_data:additional_data,
                    labels:labels,
                    total:total,
                    count:count,
                    date:date,
                    limit:limit
                });
            });
            
        }
    }
})

.factory('weekOverview', function($q, localStorageService, $translate, $filter) {
    var current = -1;
    var moving_data;
    var currentYear = moment().year();
    
    return { 
        refresh: function() {
            moving_data = localStorageService.get('moving_data') || {};
        },
        override: function() {
            return {
                type: 'time',
                ticks:{
                    fontSize: 8,
                    fontColor:'#878787'
                },
                time: {
                    unit:'day',
                    unitStepSize: 1,
                    displayFormats: {
                        'day': 'DD.'
                    }
                }
            }
        },
        title: function(){
            return $filter('titlize')($translate.instant("Wochenübersicht"));
        },
        current: function(){
            return current;
        },
        currentYear: function() {
            return currentYear;  
        },
        change: function(add) {
            var tmpD = moment().year(currentYear).week(current).startOf('week');
            tmpD.add(add, 'w');
            current = tmpD.week();
            currentYear = tmpD.year();
        },
        data: function(getCalories){
            return $q(function(resolve) { 
                // get current
                if (current < 0) {
                    current = moment().week();
                }

                if (!moving_data) moving_data = localStorageService.get('moving_data') || {};

                // fetchData and create labels
                var steps = [];
                var calories = [];
                var distance = [];
                var labels = [];
                var ex_amount = 0;
                var ex_time = 0;
                var total = {
                    steps:0,
                    calories:0,
                    distance:0
                };
                var count = 0;
                var tmpD = moment().year(currentYear).week(current).startOf('week');
                while (tmpD.week() == current) {
                    // push date to labels
                    labels.push(angular.copy(tmpD));
                    
                    // get week and check for date
                    var idx = tmpD.format('YYYY-MM-DD');
                    if ((day = moving_data[idx])) {
                        var d = day[day.length-1];
                        
                        total.steps += d.s;
                        steps.push(d.s);
                        total.calories += d.c;
                        calories.push(d.c);
                        total.distance += d.d;
                        distance.push(d.d);
                        ex_amount += d.ex_a;
                        ex_time += d.ex_t;
                        count++;
                    } else {
                        // no data for this day
                        steps.push(0);
                        calories.push(0);
                        distance.push(0);
                    }

                    // add one day
                    tmpD.add(1, 'd'); 
                }
                
                total.calories = Math.round(total.calories);
                total.distance = Math.round(total.distance/1000);
                var date = $translate.instant("Kalenderwoche")+" "+current+" - "+currentYear;
                
                // decide if next button should be blocked
                var limit = false;
                if (current == moment().week() && moment().year() == currentYear) {
                   limit = true;
                } 
                
                // data/additional_data
                var data,additional_data;
                if (!getCalories) {
                    data = steps;
                    additional_data = {
                        calories:calories
                    }
                } else {
                    data = calories;
                    additional_data = {
                        steps:steps
                    };
                } 
                additional_data['distance'] = distance;
                additional_data['ex_amount'] = ex_amount;
                additional_data['ex_time'] = ex_time;
                
                resolve ({
                    data:[data],
                    additional_data:additional_data,
                    labels:labels,
                    total:total,
                    count:count,
                    date:date,
                    limit:limit
                });
            });
            
        }
    }
})

.factory('dayOverview', function($q, localStorageService, $translate, $filter) {
    var current = -1;
    var moving_data;
    var currentYear = moment().year();
    
    return { 
        refresh: function() {
            moving_data = localStorageService.get('moving_data') || {};
        },
        override: function() {
            return {
                type: 'time',
                ticks:{
                    fontSize: 8
                },
                
                    format: "HH:mm",
                    unit: 'hour',
                    unitStepSize: 2,
                  time: {
                    displayFormats: {
                      'minute': 'HH', 
                      'hour': 'HH', 
                    }
                  }
            }
        },
        title: function(){
            return $filter('titlize')($translate.instant("Tagesübersicht"));
        },
        current: function(){
            return current;
        },
        currentYear: function() {
            return currentYear;  
        },
        change: function(add) {
            var tmpD = moment().year(currentYear).dayOfYear(current).startOf('day');
            tmpD.add(add, 'd');
            current = tmpD.dayOfYear();
            currentYear = tmpD.year();
        },
        data: function(getCalories){
            return $q(function(resolve) { 
                // get current
                if (current < 0) {
                    current = moment().dayOfYear();
                }

                if (!moving_data) moving_data = localStorageService.get('moving_data') || {};

                // fetchData and create labels
                var steps = [];
                var calories = [];
                var distance = [];
                var labels = [];
                var total = {
                    steps:0,
                    calories:0,
                    distance:0
                };
                var ex_amount = 0;
                var ex_time = 0;

                var tmpD = moment().year(currentYear).dayOfYear(current).startOf('day');
                var date = tmpD.format('dd, DD.MM.YY');
                var dateStr = tmpD.format('YYYY-MM-DD');
                
                // empty chart
                if (!moving_data[dateStr] || moving_data[dateStr].length == 0) {
                    labels.push(angular.copy(tmpD));
                    steps.push(0);
                    calories.push(0);
                    distance.push(0);
                }
                
                if ((day = moving_data[dateStr])) {
                    // set hour before first value on 0
                    var _tmpD = angular.copy(tmpD.startOf('day'));
                    labels.push(_tmpD.add(moving_data[dateStr][0].time, 's').startOf('h'));
                    steps.push(0);
                    calories.push(0);
                    distance.push(0);
                    
                    for (var i=0,len=moving_data[dateStr].length; i<len; i++) {
                        var _tmpD = angular.copy(tmpD.startOf('day'));
                        labels.push(_tmpD.add(day[i].time, 's'));

                        // data
                        total.steps = day[i].s;
                        steps.push(day[i].s);
                        total.calories = day[i].c;
                        calories.push(day[i].c);
                        
                        // additional data
                        total.distance = day[i].d;
                        distance.push(day[i].d);
                        ex_amount = day[i].ex_a;
                        ex_time = day[i].ex_t;
                    }
                }
                
                total.calories = total.calories.toFixed(1);
                
                // decide if next button should be blocked
                var limit = false;
                if (current == moment().dayOfYear() && moment().year() == currentYear) {
                   limit = true;
                } 
                
                resolve ({
                    data:[steps, calories],
                    additional_data:{
                        distance:distance,
                        ex_amount:ex_amount,
                        ex_time:ex_time
                    },
                    labels:labels,
                    total:total,
                    date:date,
                    limit:limit
                });
            });
            
        }
    }
})

.factory('generalOverview', function($translate){
    return {
        options: function(){
            return {
                scales: {
                    yAxes: [{
                        id: 'y-axis-1',
                        type: 'linear',
                        position: 'left',
                        ticks:{
                            display:true,
                            beginAtZero:true,
                            steps:10,
                            fontSize: 8,
                            fontColor:'#878787'
                        },
                        gridLines: {
                            display:false
                        }
                    }],
                    xAxes: []
                },
                tooltips: {
                    enabled:true,
                    callbacks:{
                        title: function(tooltipItem, data) {
                            return $translate.instant("Datum")+": "+tooltipItem[0].xLabel.format('DD.MM.YYYY');
                        }
                    }
                },
                legend: {
                    display:false
                },
                defaultFontColor:'#878787',
                responsive:true,                                  
                title: {
                    fontColor:'#878787',
                    fontSize:14,
                    display: true
                }
            };
        }
    }
});