'use strict';

/*
 *  btinit.js
 *  (factory)mambo2_init: initialize bluetooth -> promise on bluetooth status
 *  NOTE: also initializes android incoming phone call number display
 */

angular.module('btconnect').factory('mambo2_init', function ($q, $cordovaBluetoothLE, $cordovaDevice, bt_messenger, localStorageService, bt_sms_messenger, mambo2_permissions) {

    function android_permission_request() {
        return $cordovaBluetoothLE.requestPermission().then(function (success) {
            if (success.requestPermission) {
                return true;
            } else {
                return false;
            }
        }, function (error) {
            return error;
        });
    }

    // initialize bluetooth if needed
    function android_permission_check() {
        return $cordovaBluetoothLE.hasPermission().then(function (success) {
            if (!success.hasPermission) {
                return android_permission_request();
            } else {
                return true;
            }
        }, function (error) {
            return error;
        });
    }

    function initialize() {
        var q = $q.defer();

        $cordovaBluetoothLE.initialize().then(null, function (obj) {
            return q.reject(obj.message);
        }, function (obj) {
            if (obj.status == "enabled") return q.resolve(true);else return q.resolve(false);
        });

        return q.promise;
    }

    function phoneInit(address) {
        if (!window.PhoneCallTrap) {
            console.log("on call not available");
        } else {
            console.log("on call is available");
            window.PhoneCallTrap.onCall(function (state) {
                switch (state.status) {
                    case "RINGING":
                        console.log("Phone is ringing");

                        // display number on device
                        var numberStr = state.number + "";
                        if (numberStr.indexOf("+49") >= 0) {
                            numberStr = numberStr.replace("+49", "0");
                        }
                        address = address ? address : localStorageService.get('BTDevice').address;
                        if (name = state.name) {
                            bt_messenger.send(name, address);
                        } else {
                            bt_messenger.send(numberStr, address);
                        }
                        break;
                    case "OFFHOOK":
                        console.log("Phone is off-hook");
                        break;
                    case "IDLE":
                        console.log("Phone is idle");
                        break;
                    case "SMS":
                        console.log("SMS received");

                        // display number on device
                        var numberStr = state.number + "";
                        if (numberStr.indexOf("+49") >= 0) {
                            numberStr = numberStr.replace("+49", "0");
                        }
                        address = address ? address : localStorageService.get('BTDevice').address;
                        if (name = state.name) {
                            bt_sms_messenger.send(name, address, state.message);
                        } else {
                            bt_sms_messenger.send(numberStr, address, state.message);
                        }
                        break;
                }
            });
        }
    }

    function init() {
        var version = $cordovaDevice.getVersion();
        if (ionic.Platform.isAndroid()) {
            // sends phone number on call to device
            phoneInit

            // starting android 6.0 permission request is required
            ();if (parseInt(version.split(".")[0]) >= 6) {
                mambo2_permissions.initialize();
                return android_permission_check().then(function (success) {
                    if (success) return initialize();else return success; // false
                }, function (error) {
                    return error;
                });
            } else return initialize();
        } else {
            // iOS
            return initialize();
        }
    }

    function call_init(arg) {
        // bt_sms_messenger.send_title({'name':'Tobias Philipp', 'number':'017623680939'}, arg);
        if (ionic.Platform.isAndroid()) {
            // sends phone number on call to device
            phoneInit(arg);
        };
    }

    return {
        initialize: init,
        call_init: call_init
    };
});