'use strict';

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/*
 *  connection.js
 *  (factory)mambo2_connect: check if connected (now/before), then connect/reconnect
 *  (factory)mambo2_disconnect: check if connected, then disconnect, then close
 */

angular.module('btconnect').factory('mambo2_connect', function ($cordovaBluetoothLE, mambo2_util, mambo2_address) {
    var PREP_STAT = {
        FatalError: 0,
        Connect: 1,
        Reconnect: 2,
        Already: 3
    };
    var CONN_STAT = {
        FatalError: 0,
        Connected: 1,
        Error: 4
    };
    var error;
    var device;

    /*
     *  PREPARE
     */

    function isConnected() {
        // first check if device is already connected
        return $cordovaBluetoothLE.isConnected({ "address": device.address }).then(function (success_isConnected) {
            if (success_isConnected.isConnected) {
                // device is already connected
                return PREP_STAT.Already;
            } else {
                // not connected now -> check if was connected before
                return wasConnected();
            }
        }, function (error_isConnected) {
            if (error_isConnected.error == "neverConnected") {
                // was never connected -> connect now
                return PREP_STAT.Connect;
            } else {
                error = error_isConnected;
                return PREP_STAT.FatalError;
            }
        });
    };

    function wasConnected() {
        return $cordovaBluetoothLE.wasConnected({ "address": device.address }).then(function (success_wasConnected) {
            if (success_wasConnected.wasConnected) {
                // device is was connected before -> reconnect
                return PREP_STAT.Reconnect;
            } else {
                // not connected before -> connect
                return PREP_STAT.Connect;
            }
        }, function (error_wasConnected) {
            error = error_isConnected;
            return PREP_STAT.FatalError;
        });
    }

    function parsePrepare(value) {
        switch (value) {
            case PREP_STAT.Already:
                return PREP_STAT.Already;
                break;
            case PREP_STAT.Connect:
                return connect();
                break;
            case PREP_STAT.Reconnect:
                return reconnect();
                break;
            case PREP_STAT.FatalError:
                return PREP_STAT.FatalError;
                break;
        }
    }

    function prepare() {
        return isConnected().then(function (value) {
            return parsePrepare(value);
        }, function (value) {
            return parsePrepare(value);
        });
    }

    /*
     *  CONNECT
     */

    function reconnect() {
        return $cordovaBluetoothLE.reconnect({ "address": device.address }).then(null, function (error_reconnect) {
            error = error_reconnect;
            console.error("Error reconnect: " + JSON.stringify(error_reconnect));
            return connect(); //CONN_STAT.FatalError;
        }, function (success_reconnect) {
            if (success_reconnect.status == "connected") {
                return CONN_STAT.Connected;
            } else {
                error = success_reconnect;
                return CONN_STAT.Error;
            }
        });
    }

    function connect() {
        return $cordovaBluetoothLE.connect({ "address": device.address }).then(null, function (error_connect) {
            error = error_connect;
            return CONN_STAT.FatalError;
        }, function (success) {
            if (success.status == "connected") {
                return CONN_STAT.Connected;
            } else {
                error = success;
                return CONN_STAT.Error;
            }
        });
    }

    function parseResult(value) {
        switch (value) {
            case PREP_STAT.Already:
            case CONN_STAT.Connected:
                return {
                    status: true,
                    type: value,
                    device: device
                };
                break;
            case CONN_STAT.Error:
            case CONN_STAT.FatalError:
                return {
                    status: false,
                    type: value,
                    error: error
                };
                break;
        }
    }

    function c(d) {
        mambo2_address.set(d.address);
        device = d;

        return prepare().then(function (value) {
            return parseResult(value);
        }, function (error) {
            return parseResult(error);
        }, function (value) {
            return parseResult(value);
        });
    }

    return {
        connect: c
    };
}).factory('mambo2_disconnect', function (mambo2_address, $cordovaBluetoothLE) {
    var STAT = {
        FatalError: 0,
        Disconnected: 1,
        Reconnect: 2,
        WasNot: 3
    };
    var error;
    var connectAgain = false;

    function close() {
        return $cordovaBluetoothLE.close({ address: mambo2_address.get() }).then(function (close_success) {
            if (!connectAgain) {
                return STAT.Disconnected;
            } else {
                return STAT.Reconnect;
            }
        }, function (close_error) {
            return STAT.FatalError;
        });
    }

    function disconnect() {
        return $cordovaBluetoothLE.disconnect({ address: mambo2_address.get() }).then(function (disconnect_success) {
            return close();
        }, function (disconnect_error) {
            if (disconnect_error.error == "isDisconnected") {
                return STAT.Disconnected;
            } else return STAT.FatalError;
        });
    };

    function prepare() {
        return $cordovaBluetoothLE.isConnected({ "address": mambo2_address.get() }).then(function (success_isConnected) {
            if (success_isConnected.isConnected) {
                // device is already connected
                return disconnect();
            } else {
                // not connected now -> check if was connected before
                error = success_isConnected;
                return close(); //STAT.WasNot;
            }
        }, function (error_isConnected) {
            if (error_isConnected.error == "neverConnected") {
                // was never connected -> connect now
                return STAT.WasNot;
            } else {
                error = error_isConnected;
                return disconnect();
            }
        });
    }

    function parseDisconnectResults(value) {
        switch (value) {
            case STAT.WasNot:
            case STAT.Disconnected:
                return _defineProperty({
                    status: true,
                    reconnect: false,
                    type: value
                }, 'reconnect', connectAgain);
                break;
            case STAT.Reconnect:
                return _defineProperty({
                    status: true,
                    reconnect: true,
                    type: value
                }, 'reconnect', connectAgain);
                break;
            case STAT.FatalError:
                return {
                    status: false,
                    reconnect: connectAgain,
                    type: value,
                    error: error
                };
                break;
        }
    }

    function d(again) {
        connectAgain = again;

        return prepare().then(function (value) {
            return parseDisconnectResults(value);
        }, function (value) {
            return parseDisconnectResults(value);
        });
    }

    return {
        disconnect: d
    };
});