'use strict';

angular.module('btconnect').controller('BTConnectController', function ($scope) {
    // BT-functionality needs to be working throughout the entire app. That's why the root controller "AppCtr" is used rather than BTConnectController.
}).controller('AppCtrl', function ($scope, $ionicPlatform, $cordovaBluetoothLE, localStorageService, $mdDialog, $translate, $state, $rootScope, $ionicHistory, mambo2_util, mambo2_services, mambo2_connect, mambo2_disconnect, mambo2_scan, mambo2_init, $timeout, mambo2_screen, mambo2_compress_data, mambo2_heart_data, bt_sms_messenger) {

    $ionicPlatform.ready(function () {

        /*
         *  VARIABLES
         */

        $scope.devices = [];
        $rootScope.connectedDevice = undefined;
        $rootScope.$watch('connectedDevice', function (newVal) {
            if (newVal) {
                var displayName = "<div ng-click='namePressed(connectedDevice)'>" + newVal.name + "</div>";
                $translate('TEXT Bluetooth verbunden', { displayname: displayName }).then(function (translation) {
                    $scope.text_connected = translation;
                });
            }
        }, true);

        $rootScope.myBleIsConnected = false;
        $scope.bleIsConnecting = false;
        $scope.isScanning = false;

        /*
         *  BT STATUS
         */

        function initBTStatus() {
            cordova.plugins.BluetoothStatus.initPlugin();
            window.addEventListener('BluetoothStatus.enabled', function () {
                if (!$scope.isScanning) {
                    console.log('Bluetooth has been enabled');
                    $scope.bluetoothIsEnabled = true;

                    var refAddress = $rootScope.connectedDevice ? $rootScope.connectedDevice.address : "";
                    var refDevice = localStorageService.get('BTDevice') || {};
                    if ($state.current.name != "app.btconnect" && refAddress != refDevice.address) {
                        // redirect
                        console.log("REDIRECT");
                        $ionicHistory.nextViewOptions({
                            disableBack: true
                        });
                        $state.go('app.btconnect');
                    } else if ($rootScope.connectedDevice = localStorageService.get('BTDevice')) {
                        $scope.connect($rootScope.connectedDevice);
                    }
                }
            });
            window.addEventListener('BluetoothStatus.disabled', function () {
                console.log('Bluetooth has been disabled');
                $scope.bluetoothIsEnabled = false;

                $scope.devices = [];
                $timeout(function () {
                    $rootScope.myBleIsConnected = false;

                    $rootScope.$broadcast('btConnection', { state: 2 });
                });
                $scope.isScanning = false;
            });
        }

        /*
         *  INIT
         */

        var init = $scope.$on('btstart', function (e) {
            console.log("INIT");
            mambo2_init.initialize().then(function (success) {
                $scope.bluetoothIsEnabled = success;
                if (!$scope.bluetoothIsEnabled) {
                    $ionicHistory.nextViewOptions({
                        disableBack: true
                    });
                    $state.go('app.btconnect');
                } else {
                    $scope.btListenerBlock = false;
                    var prefAvailable = ($rootScope.connectedDevice = localStorageService.get('BTDevice')) ? true : false;

                    if (prefAvailable) {
                        $scope.connect($rootScope.connectedDevice);
                    } else {
                        $scope.scanBT(null, true);
                    }
                }
                initBTStatus();
            }, function (error) {
                $scope.bluetoothIsEnabled = false;
                mambo2_util.error("Error on bluetooth status.", "controler.js:init()", error);
                initBTStatus();
            });
        });

        /*
         *  CONNECT
         */

        function evaluateConnectCallback(result) {
            mambo2_scan.stop();
            $scope.bleIsConnecting = false;
            if (result.status) {
                // save the device
                $rootScope.connectedDevice = result.device;
                localStorageService.set("BTDevice", result.device);
                mambo2_init.call_init(result.device.address);
                // ask to rename
                askToRename(result.device, true);

                mambo2_services.init().then(function () {
                    bt_sms_messenger.subscribe(result.device.address);
                }, function (error) {
                    mambo2_util.error("Error services subscribe", "mambo2_services:subscribe()", error);
                });
                $cordovaBluetoothLE.stopScan();

                // feedback to GUI
                $rootScope.myBleIsConnected = true;
                $rootScope.$broadcast("btConnection", { state: 1 });

                // init screen settings
                mambo2_screen.init();
            } else {
                mambo2_util.error("Error connect", "mambo2_connect:c()", result.error);
                $rootScope.bleIsConnecting = false;
                if (result.error.status == "disconnected" && $rootScope.connectedDevice) {
                    $scope.connect($rootScope.connectedDevice);
                } else if (result.error.error == "isNotDisconnected") {
                    mambo2_services.reset(function (success) {
                        init();
                    }, function (error) {
                        $scope.connect($rootScope.connectedDevice);
                    });
                } else if (result.error.error == "reconnect" || result.error.error == "connect") {
                    $scope.disconnect(true);
                } else {
                    console.error("Unknown error: " + JSON.stringify(result.error));
                }
            }
        }

        $scope.connect = function (device) {
            $scope.bleIsConnecting = true;
            mambo2_connect.connect(device).then(function (success) {
                evaluateConnectCallback(success);
            }, function (error) {
                evaluateConnectCallback(error);
            }, function (success) {
                evaluateConnectCallback(success);
            });
        };

        /*
         *  DISCONNECT
         */

        function evaluateDisonnectCallback(value) {
            if (value.status) {
                $rootScope.myBleIsConnected = false;
                $rootScope.$broadcast("btConnection", { state: 2 });
                if (value.reconnect) {
                    $timeout(function () {
                        $scope.connect($rootScope.connectedDevice);
                    }, 500);
                } else {
                    console.warn("Prefered device (-> local storage: BTDevice) is being removed!");
                    localStorageService.remove('BTDevice');
                    $scope.scanBT();
                }
            } else {
                mambo2_util.error("Error disconnect", "mambo2_disconnect:disconnect()", value.error);
                //                if (value.reconnect) {
                //                    $scope.connect($rootScope.connectedDevice);
                //                } else {
                //                    console.warn("Prefered device (-> local storage: BTDevice) is being removed!");
                //                    localStorageService.remove('BTDevice');
                //                    $scope.scanBT();
                //                }
            }
        }

        $scope.disconnect = function () {
            var connectAgain = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

            if (!$rootScope.connectedDevice) {
                mambo2_util.error("Error disconnect", "controller.js:$scope.disconnect()", { message: "No connected device!" });
                return;
            }

            $scope.bleIsConnecting = false;
            mambo2_disconnect.disconnect(connectAgain).then(function (success) {
                evaluateDisonnectCallback(success);
            }, function (error) {
                evaluateDisonnectCallback(error);
            });
        };

        /*
         *  SCAN
         */
        $scope.$watch(function () {
            return mambo2_scan.devices;
        }, function (newV, oldV) {
            if (newV != oldV) {
                $scope.devices = [];
                $scope.devices = newV;
                console.log("TEST: " + JSON.stringify($scope.devices));
            }
        }, true);
        $scope.$watch(function () {
            return mambo2_scan.isScanning;
        }, function (newV, oldV) {
            if (!newV && oldV) console.log("scan finished.");
            if (newV != oldV) $scope.isScanning = newV;
        }, true);

        $scope.scanBT = function (btdevice) {
            var fetchPrefered = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
            // test ignore btdevice & fetchprefered and always connect
            btdevice = localStorageService.get('BTDevice') || {};

            if ($scope.isScanning) mambo2_scan.stop();else {
                $scope.isScanning = true;
                $scope.devices = [];
                mambo2_scan.scan(btdevice).then(function (prefered) {
                    console.log("PREFERED: " + JSON.stringify(prefered));
                    $scope.connect(prefered); //mambo2_connect.connect(prefered);
                }, function (error) {
                    console.log("SCANERROR: " + JSON.stringify(error));
                    if (error.error != "startScan") mambo2_util.error("Scan error.", "AppCtrl:scanBT()", error);else {
                        mambo2_scan.stop().then(function (success) {
                            $scope.scanBT($rootScope.connectedDevice);
                        }, function (error) {
                            mambo2_util.error("Scan error.", "AppCtrl:scanBT()", error);
                            $scope.scanBT($rootScope.connectedDevice);
                        });
                    }
                });
            }
        };

        /*
         *  RENAME
         */

        var askToRename = function askToRename(device, onlyNew) {
            var devices = localStorageService.get("knownBTDevices") || [];
            var idx = -1;
            for (var i = 0; i < devices.length; i++) {
                if (devices[i].address == device.address) {
                    idx = i;
                }
            }

            var promptMessage = {};
            promptMessage['title'] = $translate.instant('Verbindung benennen');
            promptMessage['ok'] = $translate.instant('OK');
            promptMessage['cancel'] = $translate.instant('abbrechen');
            promptMessage['text'] = $translate.instant('TEXT Gerät benennen');

            if (!onlyNew || onlyNew && idx < 0) {
                var prompt = $mdDialog.prompt().title(promptMessage.title).textContent(promptMessage.text).placeholder(device.name).ariaLabel(promptMessage.title).initialValue(device.name).ok(promptMessage.ok).cancel(promptMessage.cancel);
                $mdDialog.show(prompt).then(function (result) {
                    device.name = result;
                    if (idx >= 0) {
                        devices[idx] = device;
                    } else {
                        devices.push(device);
                    }
                    localStorageService.set("knownBTDevices", devices);
                    $rootScope.connectedDevice = device;
                });
            }
        };

        /*
         *  IONIC EVENTS
         */

        $scope.$on('$ionicView.enter', function () {
            if (!$rootScope.myBleIsConnected && $scope.bluetoothIsEnabled) {
                $scope.scanBT(null, true);
            }
        });

        $scope.$on('$ionicView.leave', function () {
            if (!$rootScope.myBleIsConnected) {
                $cordovaBluetoothLE.stopScan();
            }
        });

        $scope.namePressed = function (device) {
            askToRename(device);
        };
    }); // $ionicPlatform.ready - end
}).directive('compile', ['$compile', function ($compile) {
    return function (scope, element, attrs) {
        scope.$watch(function (scope) {
            return scope.$eval(attrs.compile);
        }, function (value) {
            element.html(value);
            $compile(element.contents())(scope);
        });
    };
}]);