'use strict';

/*
 *  login.js 
 *  (factory) mambo2_login_responder: creates hexstring for login response
 *  (factory) mambo2_login: send response to login
 */

angular.module('btconnect').factory('mambo2_login', function ($cordovaBluetoothLE, mambo2_util, mambo2_address, mambo2_login_responder, mambo2_message_reveiver, mambo2_message_sender) {
    function login() {
        var subParams = {
            "address": mambo2_address.get(),
            "service": "A500",
            "characteristic": "A501"
        };
        $cordovaBluetoothLE.subscribe(subParams).then(null, function (error) {
            mambo2_util.error("Subscription to A501 failed.", "mambo2_connect:login()", error);
            if (error.error === "subscription") {
                $cordovaBluetoothLE.unsubscribe(subParams).then(function (success) {
                    mambo2_util.success("Unsubscribe to A501 (-> subscribe again)", success);
                    login();
                }, function (error) {
                    mambo2_util.error("Subscription to A501 failed.", "mambo2_connect:login()", error);
                });
            }
        }, function (success) {
            mambo2_util.success("Read", success);

            var val = success.value;
            if (val != null) {
                mambo2_message_reveiver.parse(val).then(function (success) {
                    if (success.status) {
                        // write response
                        var responseValue = mambo2_login_responder.response();
                        console.log("PACKAGE: " + responseValue);
                        var encodedString = mambo2_message_sender.send("8001", responseValue);
                    }
                });
            }
        });
    };

    return {
        login: login
    };
}).factory('mambo2_login_responder', function (api_version, mambo2_util, localStorageService) {

    var uTCtoHex = function uTCtoHex() {
        var sec = moment().unix();
        return mambo2_util.padStr(sec.toString(16), 8);
    };

    var timezone = function timezone() {
        var offset = moment().utcOffset();

        var test = -720;
        var count = 0;
        while (offset != test) {
            test += 15;
            count += 1;
        }

        var tz = mambo2_util.padStr(count.toString(16), 2);
        localStorageService.set('timezone', tz);
        return tz;
    };

    var system = function system() {
        if (ionic.Platform.isIOS()) return "01";else if (ionic.Platform.isAndroid()) return "02";else if (ionic.Platform.isWindowsPhone()) return "03";
    };

    var response = function response() {
        var r = "";

        r += "01"; // login success
        r += uTCtoHex(); // UTC
        r += timezone(); // timezone
        r += system(); // system (iOS,Android,QP)
        return r;
    };

    return {
        response: response
    };
});