'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Message = function () {
    //    frames;
    //    message;
    //    dataLength;
    //    api;
    //    pSerial;
    function Message(api) {
        _classCallCheck(this, Message);

        this.api = api;
        this.frames = [];
        this.dataLength = 0;
    }

    _createClass(Message, [{
        key: "addToFrames",
        value: function addToFrames(frame) {
            var found = this.frames.find(function (value) {
                return value.serial == frame.serial;
            });
            if (!found) {
                this.frames.push(frame);
                if (this.frames.length == this.message.amountOfFrames) {
                    this.combineFrames();
                    return true;
                }
            }
            return false;
        }

        // return boolean -> true when message complete

    }, {
        key: "firstFrame",
        value: function firstFrame(frame) {
            // package serial
            var firstFrameLength = 40;
            this.pSerial = frame.substring(0, 4);
            firstFrameLength -= 4;

            // length
            var length = frame.substring(4, 6);
            firstFrameLength -= 2;
            var l = parseInt(length, 16) * 2; // 2char for each byte

            // api
            var api = frame.substring(8, 12);
            if (api != this.api) console.error("Message class error:\nWrong api. Expected '" + this.api + "', but got '" + api + "'\n");
            l -= 4;
            firstFrameLength -= 4;

            // frame (serial & data)
            var fSerial = parseInt(frame.substring(6, 8)); // should always be "01" on the first frame
            firstFrameLength -= 2;
            var data = "";
            if (l <= firstFrameLength) {
                data += frame.substring(12, 12 + parseInt(length, 16) * 2);
            } else {
                data += frame.substring(12);
            }

            // amount of frames
            l -= data.length; // remaining length
            var amountOfFrames = Math.ceil(l / 38) + 1; // each other frame got serial (2char) and data (38char)
            this.dataLength = l;

            this.message = {
                pSerial: this.pSerial,
                amountOfFrames: amountOfFrames,
                data: "",
                api: api
            };

            return this.addToFrames({
                serial: fSerial,
                data: data
            });
        }

        // return boolean -> true when message complete

    }, {
        key: "otherFrame",
        value: function otherFrame(frame) {
            var fSerial = parseInt(frame.substring(0, 2));
            var data = frame.substring(2);

            // last frame
            var amount = this.message.amountOfFrames;
            if (fSerial == amount && amount > 1) {
                // serials start "01"
                var totalLength = (amount - 1) * 38;
                var rest = 38 - (totalLength - this.dataLength);
                data = data.substring(0, rest);
            }

            return this.addToFrames({
                serial: fSerial,
                data: data
            });
        }
    }, {
        key: "addData",
        value: function addData(data) {
            var result = void 0;
            if (parseInt(data.substring(0, 2), 16) >= 128) {
                result = this.firstFrame(data);
            } else {
                result = this.otherFrame(data);
            }

            if (result) {
                this.message['status'] = true;
                return this.message;
            } else return false;
        }
    }, {
        key: "combineFrames",
        value: function combineFrames() {
            // sort by serial
            this.frames.sort(function (a, b) {
                var serialA = a.serial;
                var serialB = b.serial;
                if (serialA < serialB) {
                    return -1;
                }
                if (serialA > serialB) {
                    return 1;
                }
                return 0;
            });

            // combine
            var data = "";
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = this.frames[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var frame = _step.value;

                    data += frame.data;
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            this.message.data = data;
        }
    }]);

    return Message;
}();

angular.module('btconnect').factory('mambo2_message', function (api_version, mambo2_util) {

    function buildPackage(pSerial, value) {
        var response = {
            "package": pSerial,
            "value": value
        };

        // add version to data
        value = api_version.get() + value;

        // get length in byte, including crc32 (2 char per byte)
        var length = (value.length / 2 + 4).toString(16);
        length = mambo2_util.padStr(length + "", 2);

        // get crc32
        var crc32 = getCRCResult(value);

        if (value.length <= 24) {
            // 1 frame
            response["framesNum"] = 1;
            var frame = pSerial + length + "01" + value + crc32;
            frame = fillFrame(frame);
            response["decoded"] = [frame];
            var encoded = encodeFrame(frame);
            response["frames"] = [encoded];
        } else {
            // multiple frames
            var frames = new Array();
            var decoded = new Array();
            var frameIdx = 1;
            while (value.length > 0) {
                var frame = "";
                if (frameIdx == 1) {
                    frame += pSerial + length + "01" + value;
                    if (frame.length > 40) {
                        frame = frame.substring(0, 40);
                        var removeNum = frame.substring(8).length;
                        value = value.substring(removeNum);
                    } else {
                        value = "";
                        var diff = 40 - frame.length;
                        frame += crc32.substr(0, diff);
                        crc32 = crc32.substr(diff);
                        frame = fillFrame(frame);
                    }
                } else {
                    var idx = ("0" + frameIdx).slice(-2);
                    if (value.length == 0) {
                        frame += idx + crc32;
                        frame = fillFrame(frame);
                    } else {
                        frame += idx + value;
                        if (frame.length > 40) {
                            frame = frame.substring(0, 40);
                            var removeNum = frame.substring(8).length;
                            value = value.substring(removeNum);
                        } else {
                            value = "";
                            var diff = 40 - frame.length;
                            frame += crc32.substr(0, diff);
                            crc32 = crc32.substr(diff);
                            frame = fillFrame(frame);
                        }
                    }
                }
                decoded.push(frame);
                var encoded = encodeFrame(frame);
                frames.push(encoded);
                frameIdx++;
            }
            response["decoded"] = decoded;
            response["frames"] = frames;
            response["framesNum"] = frameIdx - 1;
        }
        return response;
    }

    return {
        buildPackage: buildPackage
    };
});