"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*
 *  message_receiver.js
 *  (class) Message: collects frame data and combines it to one message. 
 *  (factory) mambo2_message_receiver<promise>: wraps/instantiates Message class and keeps track of package id and provides feedback
 */

var Message = function () {
    function Message(api) {
        _classCallCheck(this, Message);

        this.api = api;
        this.frames = [];
        this.dataLength = 0;
    }

    _createClass(Message, [{
        key: "addToFrames",
        value: function addToFrames(frame) {
            var found = this.frames.find(function (value) {
                return value.serial == frame.serial;
            });
            if (!found) {
                this.frames.push(frame);
                if (this.frames.length == this.message.amountOfFrames) {
                    this.combineFrames();
                    return true;
                }
            }
            return false;
        }

        // return boolean -> true when message complete

    }, {
        key: "firstFrame",
        value: function firstFrame(frame) {
            // package serial
            var firstFrameLength = 40;
            this.pSerial = frame.substring(0, 4);
            firstFrameLength -= 4;

            console.log("/-------------" + this.pSerial + "-------------/");
            console.log(frame);

            // length
            var length = frame.substring(4, 6);
            firstFrameLength -= 2;
            var l = parseInt(length, 16) * 2; // 2char for each byte

            // api
            var api = frame.substring(8, 12);
            if (api != this.api) console.error("Message class error:\nWrong api. Expected '" + this.api + "', but got '" + api + "'\n");
            l -= 4;
            firstFrameLength -= 4;

            // frame (serial & data)
            var fSerial = parseInt(frame.substring(6, 8)); // should always be "01" on the first frame
            firstFrameLength -= 2;
            var data = "";
            if (l <= firstFrameLength) {
                data += frame.substring(12, 12 + parseInt(length, 16) * 2);
            } else {
                data += frame.substring(12);
            }

            // amount of frames
            l -= data.length; // remaining length
            var amountOfFrames = Math.ceil(l / 38) + 1; // each other frame got serial (2char) and data (38char)
            this.dataLength = l;

            this.message = {
                pSerial: this.pSerial,
                amountOfFrames: amountOfFrames,
                data: "",
                api: api
            };

            return this.addToFrames({
                serial: fSerial,
                data: data
            });
        }

        // return boolean -> true when message complete

    }, {
        key: "otherFrame",
        value: function otherFrame(frame) {
            console.log(frame);
            var fSerial = parseInt(frame.substring(0, 2));
            var data = frame.substring(2);

            // last frame
            var amount = this.message.amountOfFrames;
            if (fSerial == amount && amount > 1) {
                // serials start "01"
                var totalLength = (amount - 1) * 38;
                var rest = 38 - (totalLength - this.dataLength);
                data = data.substring(0, rest);
            }

            return this.addToFrames({
                serial: fSerial,
                data: data
            });
        }
    }, {
        key: "addData",
        value: function addData(data) {
            var result = void 0;
            if (parseInt(data.substring(0, 2), 16) >= 128) {
                result = this.firstFrame(data);
            } else {
                result = this.otherFrame(data);
            }

            if (result) {
                this.message['status'] = true;

                console.log("RECEIVED: " + JSON.stringify(this.message));
                return this.message;
            } else return false;
        }
    }, {
        key: "combineFrames",
        value: function combineFrames() {
            // sort by serial
            this.frames.sort(function (a, b) {
                var serialA = a.serial;
                var serialB = b.serial;
                if (serialA < serialB) {
                    return -1;
                }
                if (serialA > serialB) {
                    return 1;
                }
                return 0;
            });

            // combine
            var data = "";
            angular.forEach(this.frames, function (frame) {
                data += frame.data;
            });

            this.message.crc32 = data.substring(data.length - 8);
            this.message.data = data.substring(0, data.length - 8);
        }
    }]);

    return Message;
}();

angular.module('btconnect').factory('mambo2_message_reveiver', function ($cordovaBluetoothLE, $q, mambo2_util, api_version, busy) {
    var pSerial = 0;
    var activeMessage = {};

    var encodedToByteString = function encodedToByteString(input) {
        var val = $cordovaBluetoothLE.encodedStringToBytes(input);
        var byteStr = "";
        for (var i = 0; i < val.length; i++) {
            var byte = val[i].toString(16);
            if (byte.length == 1) byte = "0" + byte;
            byteStr += byte;
        }
        return byteStr;
    };

    function parse(input) {
        var result = {
            status: false
        };
        var q = $q.defer();

        var byteStr = encodedToByteString(input);

        if (parseInt(byteStr.substring(0, 2), 16) >= 128) {
            if (byteStr.substring(0, 4) != activeMessage.pSerial) {
                busy.do(byteStr.substring(0, 4));
                if (byteStr.substring(0, 4) == "8001") {
                    var api = byteStr.substring(8, 12);
                    api_version.set(api);
                }
                activeMessage = new Message(api_version.get());
                result = activeMessage.addData(byteStr);
            } else {
                mambo2_util.warning("First frame of " + byteStr.substring(0, 4) + " has been sent twice. Check response and bt connection.", "bt_hexdata:parse()");
            }
        } else if (activeMessage) {
            result = activeMessage.addData(byteStr);
        } // when a second frame is sent first it gets discarded
        if (result.status) {
            activeMessage = {};
            result['crc32Check'] = getCRCResult(result.api + result.data) == result.crc32;
        }
        q.resolve(result);
        return q.promise;
    };

    return {
        parse: parse
    };
});