"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*
 *  message_sender.js
 *  (factory) mambo2_message_builder<obj>: combines a hexstring and pSerial into an object containing all frames for this message
 */

var Sender = function () {
    function Sender(cordovaBluetoothLE, mambo2_address, mambo2_util, busy) {
        _classCallCheck(this, Sender);

        this.mambo2_util = mambo2_util;
        this.cordovaBluetoothLE = cordovaBluetoothLE;
        this.mambo2_address = mambo2_address;
        this.mambo2_util = mambo2_util;
        this.busy = busy;
        this.loopIndex = 0;
        this.params = {
            "service": "A500",
            "characteristic": "A502",
            "address": this.mambo2_address.get()
        };
    }

    _createClass(Sender, [{
        key: "writeMultipleFrames",
        value: function writeMultipleFrames() {
            var self = this;
            (function fn() {
                if (self.loopIndex < self.loopPackage.framesNum) {
                    console.log("PACKAGE SEND: " + self.loopIndex + " -> " + self.loopPackage.frames[self.loopIndex]);
                    self.params["value"] = self.loopPackage.frames[self.loopIndex];
                    self.loopIndex++;

                    self.cordovaBluetoothLE.write(self.params).then(function (success) {
                        fn();
                    }, function (error) {
                        self.mambo2_util.error("Error writing to A502.", "mambo2_message_sender:writeMultipleFrames()", error);
                    });
                } else {
                    self.busy.do();
                }
            })();
        }
    }, {
        key: "sendPackage",
        value: function sendPackage(packageObj) {
            this.loopPackage = packageObj;
            this.writeMultipleFrames();
        }
    }]);

    return Sender;
}();

angular.module('btconnect').factory('mambo2_message_builder', function (api_version, mambo2_util, $cordovaBluetoothLE) {
    var encodeFrame = function encodeFrame(frame) {
        var bytes = hexStringToBinary(frame);
        return $cordovaBluetoothLE.bytesToEncodedString(bytes);
    };

    function buildPackage(pSerial, value) {
        var response = {};

        // add version to data
        value = api_version.get() + value;

        // get length in byte, without crc32 and <package><length><frame_idx> (2 char per byte) 
        var length = (value.length / 2 + 4).toString(16);
        length = mambo2_util.padStr(length, 2);
        response['data'] = {
            value: value,
            length: length
        };

        // get crc32
        var crc32 = getCRCResult(value);
        response['crc32'] = crc32;

        var frame = pSerial + length + value + crc32; // the frame idx "01" is omitted here
        response['package'] = {
            serial: pSerial,
            value: frame
        };

        var frames = new Array();
        var decoded = new Array();
        var amountOfFrames = Math.ceil(frame.length / 38);
        response["framesNum"] = amountOfFrames;

        for (var i = 0; i < amountOfFrames; i++) {
            var tmp = frame.substr(i * 38, 38);
            var idx = mambo2_util.padStr((i + 1).toString(16), 2);
            tmp = i > 0 ? idx + tmp : tmp.substring(0, 6) + idx + tmp.substr(6);
            tmp = mambo2_util.padStr(tmp, 40, true);
            decoded.push(tmp);
            frames.push(encodeFrame(tmp));
        }

        response['decoded'] = decoded;
        response['frames'] = frames;

        return response;
    }

    return {
        buildPackage: buildPackage
    };
}).factory('mambo2_message_sender', function (mambo2_message_builder, $cordovaBluetoothLE, mambo2_address, mambo2_util, busy, $interval) {
    var sending_queue = [];

    function _send(packageObj) {
        var sender = new Sender($cordovaBluetoothLE, mambo2_address, mambo2_util, busy);
        console.log("Package: " + JSON.stringify(packageObj));
        sender.sendPackage(packageObj);
    }

    function send(pSerial, value) {
        var packageObj = mambo2_message_builder.buildPackage(pSerial, value);
        var busy_with = busy.is();
        if (!busy_with || busy_with == pSerial) {
            _send(packageObj);
        } else {
            console.log("Queue: " + JSON.stringify(packageObj));
            sending_queue.push(packageObj);
            if (sending_queue.length > 2) console.warn("Message sender queue contains more than two objects!");
            if (sending_queue.length == 1) {
                // interval needs to be initiated
                var interval = $interval(function () {
                    if (!busy.is()) {
                        _send(sending_queue[sending_queue.length - 1]);
                        sending_queue.pop();
                        if (sending_queue.length == 0) $interval.cancel(interval);
                    }
                }, 100);
            }
        }
    }
    return {
        send: send
    };
});