'use strict';

/*
 *  scan.js 
 *  (factory) mambo2_scan: scans for devices and returns them to the controller
 */

angular.module('btconnect').factory('mambo2_scan', function ($cordovaBluetoothLE, localStorageService, $q, $timeout) {
    var m = {
        q: undefined,
        error: undefined,
        devices: [],
        preferedDevice: undefined,
        knownDevices: [],
        isScanning: false,
        addToDevices: function addToDevices(d) {
            // add a scanned device to the list of available devices
            // check for name
            for (var j = 0; j < m.knownDevices.length; j++) {
                if (m.knownDevices[j].address === d.address) {
                    d = m.knownDevices[j];
                    break;
                }
            }

            // check for duplicates
            var found = false;
            for (var i = 0; i < m.devices.length; i++) {
                if (m.devices[i].address === d.address) {
                    found = true;
                    break;
                }
            }

            // add to devices displayed
            if (!found) m.devices.push(d);
        },
        paired: function paired() {
            $cordovaBluetoothLE.isScanning().then(function (success) {
                if (success.isScanning) {
                    $cordovaBluetoothLE.stopScan().then(function (success) {
                        m.retrieve();
                    }, function (error) {
                        m.retrieve();
                    });
                } else {
                    m.retrieve();
                }
                m.isScanning = success.isScanning;
            });
        },
        retrieve: function retrieve() {
            // get all paired devices
            $cordovaBluetoothLE.retrieveConnected({ services: ['A500'] }).then(function (retrieve_success) {
                console.log("Paired devices: " + JSON.stringify(retrieve_success));
                for (var i = 0; i < retrieve_success.length; i++) {
                    m.addToDevices(retrieve_success[i]);
                    if (m.preferedDevice) {
                        if (m.preferedDevice.address == retrieve_success[i].address) {
                            console.log("prefered");
                            $cordovaBluetoothLE.stopScan();
                            m.q.resolve(m.preferedDevice);
                        }
                    }
                }
                // now to unpaired
                m.scanNow(true);
            }, function (retrieve_error) {
                m.error = retrieve_error;
                m.q.reject(m.error);
            });
        },
        scanNow: function scanNow() {
            // get all unpaired devices
            $cordovaBluetoothLE.startScan({ allowDuplicates: false, services: ['A500'], scanTimeout: 10000 }).then(null, function (scan_error) {
                m.scanCheck();
                m.error = scan_error;
                m.q.reject(m.error);
            }, function (success) {
                m.scanCheck();
                if (success.advertisement && success.advertisement.isConnectable === false) {
                    // not connectable
                    console.log("nothing");
                } else if (success.status == "scanStarted") {
                    // scan started
                    m.isScanning = true;
                    console.log("started");
                } else if (success.status == "scanResult") {
                    //Device found
                    console.log("found");
                    m.addToDevices(success);
                    if (m.preferedDevice) {
                        if (m.preferedDevice.address == success.address) {
                            console.log("prefered");
                            $cordovaBluetoothLE.stopScan();
                            m.q.resolve(m.preferedDevice);
                        }
                    }
                }
            });
        },
        scanCheck: function scanCheck() {
            $cordovaBluetoothLE.isScanning().then(function (success) {
                m.isScanning = success.isScanning;
                if (m.isScanning) {
                    $timeout(function () {
                        m.scanCheck();
                    }, 1000);
                }
            });
        },
        stop: function stop() {
            return $cordovaBluetoothLE.stopScan();
        },
        scan: function scan(pref) {
            m.q = $q.defer();

            m.devices = [];
            m.knownDevices = localStorageService.get("knownBTDevices") || [];
            if (pref) {
                m.preferedDevice = pref;
            }
            m.paired();

            return m.q.promise;
        }
    };

    return m;
});