'use strict';

/*
 *  services.js
 *  
 */

angular.module('btconnect').factory('mambo2_services', function ($cordovaBluetoothLE, mambo2_util, mambo2_address, mambo2_login_responder, mambo2_message_reveiver, mambo2_message_sender, mambo2_userinfo, mambo2_login, bt_parser, $q, mambo2_deviceinfo) {
    var q;

    function parse(val) {
        mambo2_message_reveiver.parse(val).then(function (success) {
            if (success.status) {
                if (success.pSerial == "8003") {
                    mambo2_userinfo.send();
                    mambo2_deviceinfo.parse(success.data);
                } else {
                    var response_code = success.crc32Check ? "01" : "00";
                    mambo2_message_sender.send(success.pSerial, success.data.substring(0, 2) + response_code);
                    bt_parser.all(success);
                }
            }
        }, function (error) {
            mambo2_util.error("Error parsing A503 data.", "mambo2_services:parse()", error);
        });
    }

    function subscribe() {
        var params = {
            "address": mambo2_address.get(),
            "service": "A500",
            "characteristic": "A503"
        };
        $cordovaBluetoothLE.subscribe(params).then(null, function (error) {
            mambo2_util.error("Error subscribing to A503", "mambo2_services:init()", error);
            if (error.error === "subscription") {
                $cordovaBluetoothLE.unsubscribe(params).then(function (success) {
                    mambo2_util.success("Unsubscribe to A503 (-> subscribe again)", success);
                    subscribe();
                }, function (error) {
                    mambo2_util.error("Subscription to A501 failed.", "mambo2_connect:login()", error);
                    q.reject(error);
                });
            }
        }, function (success) {
            if (success.value) parse(success.value);else {
                console.log("Successfully subscribed to A503.");
            }
            q.resolve();
        });
    }

    function discovered() {
        mambo2_login.login();
        subscribe();
    }

    function discover() {
        $cordovaBluetoothLE.discover({
            "address": mambo2_address.get(),
            "services": []
        }).then(function (success) {
            mambo2_util.success("Discover", success);
            discovered();
        }, function (error) {
            mambo2_util.error("Error discovering services.", "mambo2_services:init()", error);
        });
    }

    function init() {
        q = $q.defer();

        $cordovaBluetoothLE.isDiscovered({ address: mambo2_address.get() }).then(function (success) {
            if (success.isDiscovered) {
                discovered();
            } else {
                discover();
            }
        }, function (error) {
            discover();
        });

        return q.promise;
    }

    function pause() {
        var params = {
            "address": mambo2_address.get(),
            "service": "A500",
            "characteristic": "A503"
        };
        return $cordovaBluetoothLE.unsubscribe(params).then(function (success) {
            return success.status === "unsubscribed" ? true : false;
        }, function (error) {
            mambo2_util.error("Error unsubscribing to A503", "mambo2_services:pause()", error);
        });
    }

    function reset() {
        var params = {
            "address": mambo2_address.get(),
            "service": "A500",
            "characteristic": "A501"
        };
        return $cordovaBluetoothLE.unsubscribe(params).then(function (success) {
            mambo2_util.success("Unsubscribe to A501", success);
        }, function (error) {
            mambo2_util.error("Error unsubscribing to A501", "mambo2_services:reset()", error);
        });
    }

    return {
        resume: subscribe,
        pause: pause,
        init: init,
        reset: reset
    };
});