'use strict';

/*
 *  userinfo.js
 *  (factory)mambo2_userinfo: send the userinfo to device
 */

angular.module('btconnect').factory('mambo2_userinfo', function (localStorageService, mambo2_util, mambo2_message_sender) {
    var uTCtoHex = function uTCtoHex() {
        var sec = moment().unix();
        return mambo2_util.padStr(sec.toString(16), 8);
    };

    // requires string separated by a dot. e.g.: "75.38"kg 
    var weightToHex = function weightToHex(kilo) {
        var idx = kilo.indexOf(".");
        var powerTen = 0;
        if (idx < 0) powerTen = "00";else {
            var tmp = kilo.length - (idx + 1);
            tmp = 256 - tmp;
            powerTen = tmp.toString(16);
        }

        var nums = parseInt(kilo.replace(".", ""), 10);
        var numsHex = nums.toString(16);
        numsHex = ("00000" + numsHex).slice(-6);+console.log("WEIGHT: " + kilo + " -> " + (powerTen + numsHex));
        return powerTen + numsHex;
    };

    var targetState = function targetState() {
        return "01";
    };

    var weekTarget = function weekTarget() {
        return "0000007d"; // 200 Schritte
    };

    var stepCountChange = function stepCountChange() {
        return "000000";
    };

    var boolToHex = function boolToHex(boolVal) {
        return boolVal ? "01" : "00";
    };

    var pairingState = function pairingState(state) {
        switch (state) {
            case "normal":
                {
                    return "00";
                }
            case "pairing":
                {
                    return "01";
                }
        }
        return "00";
    };

    // requires format "hh:mm"
    var timeToHex = function timeToHex(time) {
        var timeArr = time.split(":");
        var hh = parseInt(timeArr[0], 10).toString(16);
        hh = ("0" + hh).slice(-2);
        var mm = parseInt(timeArr[1], 10).toString(16);
        mm = ("0" + mm).slice(-2);
        return "" + hh + mm;
    };

    // assemble and send
    function send() {
        var pSerial = "8003";

        var cw = "50";
        var utc = uTCtoHex();

        // weight
        var tmpWeight, weight;
        if (tmpWeight = localStorageService.get('userinfo_weight')) {
            weight = weightToHex(tmpWeight);
        } else {
            localStorageService.set('userinfo_weight', "60");
            weight = weightToHex("60");
        }

        // height
        var tmpHeight, height;
        if (tmpHeight = localStorageService.get("userinfo_height")) {
            height = weightToHex(tmpHeight); // 'weightToHex' works for heights also
        } else {
            localStorageService.set('userinfo_height', "1.72");
            height = weightToHex("1.72"); // 'weightToHex' works for heights also
        }

        var ts = targetState();
        var wt = weekTarget();
        var ssc = stepCountChange();
        var erase = boolToHex(false);
        var ps = pairingState("normal");
        var heart = boolToHex(true);
        var start = timeToHex("23:20");
        var end = timeToHex("7:20");

        var value = cw + utc + weight + height + ts + wt + ssc + erase + ps + heart + start + end;

        mambo2_message_sender.send(pSerial, value);
    };

    return {
        send: send
    };
});