'use strict';

/*
 *  utils.js
 *  (factory)api_version: global holder of the api version 
 *  (factory)mambo2_address: global holder of the connected device address
 *  (factory)mambo2_util: logging utils, padStr()
 */

angular.module('btconnect').factory('api_version', function () {
    var version = "";
    return {
        set: function set(val) {
            version = val;
        },
        get: function get() {
            return version;
        }
    };
}).factory('busy', function ($q) {
    var busy = undefined;
    return {
        do: function set(val) {
            busy = val;
        },
        is: function get() {
            return busy;
        }
    };
}).factory('mambo2_address', function () {
    var address;

    return {
        set: function set(val) {
            address = val;
            console.log("The bluetooth target address was changed to " + address + ".");
        },
        get: function get() {
            return address;
        }
    };
}).factory('mambo2_util', function () {
    var padStr = function padStr(str, num, back) {
        if (str.length > num) {
            this.error("String is " + str.length + " chars long but expected less or " + num, "mambo2_util:padStr()", { string: str, fill_length: num, str_length: str.length });
        } else {
            if (!back) {
                while (str.length < num) {
                    str = "0" + str;
                }
            } else {
                while (str.length < num) {
                    str += "0";
                }
            }
        }

        return str;
    };

    var error = function error(message, position, _error) {
        if (!_error) {
            console.error("Error at " + position + ":\n" + message + "\n");
        } else {
            console.error("Error at " + position + ":\n" + message + "\n" + JSON.stringify(_error) + "\n");
        }
    };

    var warning = function warning(message, position) {
        console.warn("Warning at " + position + ":\n" + message + "\n");
    };

    var success = function success(name, obj) {
        console.log(name + "  success: " + JSON.stringify(obj));
    };

    return {
        padStr: padStr,
        error: error,
        success: success,
        warning: warning
    };
});