'use strict';

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*
 *  btdata_heart.js
 *  (factory)mambo2_heart_data
 */

var HeartData = function HeartData(data) {
    _classCallCheck(this, HeartData);

    this.day = data.day;
    this.night = data.night;
    this.all = data.data;
};

angular.module('btdata').factory('mambo2_heart_data', function (localStorageService, mambo2_compress_data, $filter) {
    var PERCENTAGE_AT_REST = 0.1; // all heart rate values ordered and then x percent of the lowest discarded

    function lowestHighestValue(array) {
        var lowestVal = 0;
        var highestVal = 0;
        var atRest = 0;

        if (array.length > 1) {
            var tmpData = $filter('orderBy')(array, 'val');
            var tenPerc = Math.floor(tmpData.length * PERCENTAGE_AT_REST);
            lowestVal = tmpData[0];

            tmpData = tmpData.slice(tenPerc);
            atRest = tmpData[0];
            highestVal = tmpData[tmpData.length - 1];
        } else if (array.length == 1) lowestVal = array[0];

        return {
            lowest: lowestVal,
            atRest: atRest,
            highest: highestVal
        };
    }

    function heartRateWithStats(date) {
        var data = mambo2_compress_data.getByDate('heart', date, { splitMode: true, sleepModeBy: 50 });
        var result = {};

        angular.forEach(data, function (val, key) {
            // key: 'data' or 'night_data'
            var tmp_data = val;

            // sort
            tmp_data.sort(function (a, b) {
                if (a.date.isBefore(b.date)) return -1;
                if (b.date.isBefore(a.date)) return 1;
                return 0;
            });

            // check last entry
            //            var last = tmp_data[tmp_data.length-1].date;
            //            if (last.minute() == 0 && last.hour() == 0 && last.day() != tmp_data[tmp_data.length-2].date.day()) {
            //                tmp_data[tmp_data.length-1].date = last.subtract(1, 'd');
            //            }

            var loHi = lowestHighestValue(tmp_data);

            var total = 0;
            var vals = [];
            var dates = [];

            angular.forEach(tmp_data, function (entry) {
                total += entry.val;
                vals.push(entry.val);
                dates.push(entry.date);
            });

            var avg = 0;
            if (tmp_data.length > 0) avg = Math.round(total / tmp_data.length);

            result[key] = {
                amountEntries: tmp_data.length,
                average: avg,
                highest: loHi.highest,
                atRest: loHi.atRest,
                lowest: loHi.lowest,
                values: vals,
                dates: dates
            };
        });

        var tmpD = moment(date).format('YYYY-MM-DD');
        if (moment().format('YYYY-MM-DD') != tmpD) {
            var at_rest = localStorageService.get('heart_rate_at_rest') || {};
            if (!at_rest[tmpD]) {
                at_rest[tmpD] = {
                    day: data.day.atRest,
                    night: data.atRest,
                    all: data.atRest
                };
                localStorageService.set('heart_rate_at_rest', at_rest);
            }
        }

        return new HeartData(result);
    }

    function heartRateAtRest(date) {
        cleanData();
        var result = undefined;

        // check already parsed data for day/night
        var local_parsed = localStorageService.get('parsed_heart_rate') || {};
        // assume local_parsed is in order by date from long ago [0] to today [n]
        var keys = Object.keys(local_parsed);
        var idx = keys.length - 1;
        var finished = false;
        while (idx >= 0 && !finished) {
            if (keys[idx] <= date) {
                var tmp = local_parsed[keys[idx]];
                if (tmp) {
                    if (tmp.day && tmp.night) {
                        if (Math.abs(tmp.day.atRest.val - tmp.night.atRest.val) > 5) {
                            // NOTE: THIS IS AN ARBITRARY VALUE
                            result = tmp;
                        }
                    }
                }
            }
            idx--;
        }

        // no parsed data (night/day) -> check if parsed "all" data for this day 
        if (!result) {
            var _tmp = local_parsed[date];
            if (_tmp) {
                if (_tmp.all) {
                    _tmp['day'] = angular.copy(_tmp.all);
                    _tmp['night'] = angular.copy(_tmp.all);
                    _tmp.night.atRest.val = _tmp.day.atRest.val - 10; // NOTE: THIS IS AN ARBITRARY VALUE
                    result = _tmp;
                }
            }
        }

        // no parsed data (all) -> check if none-parsed data for this day 
        if (!result) {
            var alt_today = localStorageService.get('alt_heart_data') || {};
            var _tmp2 = alt_today[date];
            if (_tmp2) {
                var count = 0;
                var val = 0;
                for (var t in _tmp2) {
                    val += parseInt(_tmp2[t]);
                    count++;
                }
                var avrg = Math.round(val / count);
                result = {
                    day: { atRest: { val: avrg } },
                    night: { atRest: { val: avrg - 10 } },
                    all: { atRest: { val: avrg } }
                };
            }
        }

        if (!result) {
            return {
                day: { atRest: { val: 0 } },
                night: { atRest: { val: 0 } },
                all: { atRest: { val: 0 } }
            };
        } else return result;
    }

    function cleanData() {
        var data = localStorageService.get('alt_heart_data') || {};
        var today = moment().format('YYYY-MM-DD');
        for (var h in data) {
            if (data.hasOwnProperty(h)) {
                if (h < today) {
                    var parsed = heartRateWithStats(h);
                    var local_parsed = localStorageService.get('parsed_heart_rate') || {};
                    local_parsed[h] = parsed;
                    if (localStorageService.set('parsed_heart_rate', local_parsed)) {
                        delete data[h];
                    };
                }
            }
        }
        localStorageService.set('alt_heart_data', data);
    }

    return {
        heartRateWithStats: heartRateWithStats,
        heartRateAtRest: heartRateAtRest
    };
});