angular.module('btdata.services',[])

.factory('bt_parser', function(localStorageService, btd_compress, mambo2_request_parse, mambo2_sleep_compress, mambo2_compress_data) {
    /*
     *  helper functions
     */
    function pad(num, size) {
        var s = num+"";
        while (s.length < size) s = "0" + s;
        return s;
    }
    
    var hexToFloat = function(dec, val) {
        var v = (parseInt (val, 16)).toString(10);
        var d = "";
        switch (dec.length) {
            case 1: 
                d = v.length - (16 - parseInt(dec,16));
                break;
            case 2: 
                d = v.length - (256 - parseInt(dec,16));
                break;
            case 3: 
                d = v.length - (4096 - parseInt(dec,16));
                break;
        }
        return v.insert(".",d);
    }
    
    function decodeFloat16 (binary) {
        var exponent = (binary & 0x7C00) >> 10;
            fraction = binary & 0x03FF;
        return (binary >> 15 ? -1 : 1) * (
            exponent ?
            (
                exponent === 0x1F ?
                fraction ? NaN : Infinity :
                Math.pow(2, exponent - 15) * (1 + fraction / 0x400)
            ) :
            6.103515625e-5 * (fraction / 0x400)
        );
    }; 
    
    /*
     *  parser functions
     */
    
    var pGeneric = function (val) {
        return parseInt(val, 16);
    }
    
    var pUTC = function (val) {
        return parseInt(val, 16) * 1000;
    }

    var pUTCOffset = function (val) {
        return parseInt(val, 16) * 5;
    }
    
    var pExamount = function (val) {
        var bitStr = (parseInt(val,16)).toString(2);
        return decodeFloat16 (bitStr);
    }
    
    var pCalories = function (val) {
        return hexToFloat (val.substring(0,2),val.substring(2,8));
    }
    
    var pStatusDay = function (val) {
        var status = {}
        var bitStr = (parseInt(val,16)).toString(2);
        bitStr = pad(bitStr,8);
        status['battery_level'] = parseInt(bitStr.substring(5,8),2);
        status['sleep_status'] = parseInt(bitStr.substring(3,5),2);
        status['intensity_level'] = parseInt(bitStr.substring(0,3),2);
        return status;
    }
    
    var pSleepData = function(val) {
        var data = [];
        var n = 0;
        var lengthBool = false;
        while (val.length > 0) {
            // get the right word and shorten val
            var hex = val.substr(0,2);
            val = val.substring(2);
            
            if (!lengthBool) {
                // get binary word
                var bin = (parseInt(hex,16)).toString(2);
                bin = pad(bin,8);

                // boolean
                lengthBool = (bin.substr(0,1) == "1");

                // actual sleep value
                var sleep = bin.substring(1);
                sleep = parseInt(sleep,2);
                
                var result = {
                    l:sleep
                }
                
                if (n != 0) {
                    result['lng'] = n;
                    n = 0;
                }
                
                data.push(result);
            } else {
                n = parseInt(hex,16);
                lengthBool = false;
            }
        }
        return data;
    }
    
    var pVoltage = function (val) {
        var v = parseInt(val, 16) / 100;
        return v + 1.6;
    }
    
    return {
        all: function(input) {
            var val = input.data;
            var cw = val.substring(0,2);
            var response = {};
            var response_legacy = {};
            switch (cw) {
                case "52":
                    // NEW ALGORITHM 
                    response['utc'] = pGeneric(val.substring(2,10));
                    response['utc_offset'] = pUTCOffset(val.substring(10,12));
                    response['data'] = val.substring(20);
                    var tmp_unsent = pGeneric(val.substring(12,16));
                    if (tmp_unsent > 0) response['unsent_count'] = pGeneric(val.substring(12,16));
                    response['current_count'] = pGeneric(val.substring(16,20));
                    
                    // OLD DATA COLLECTION
                    response_legacy['utc'] = pUTC(val.substring(2,10));
                    response_legacy['date'] = new Date(response_legacy['utc']).toJSON().split("T")[0];
                    response_legacy['utc_offset'] = pUTCOffset(val.substring(10,12));
                    response_legacy['unsent_count'] = pGeneric(val.substring(12,16));
                    response_legacy['current_count'] = pGeneric(val.substring(16,20));
                    response_legacy['data'] = pSleepData(val.substring(20)); 
            
                    break;
                case "51": 
                case "57":
                    var data = [];
                    var o = 0;
                    while (val.length > 0) {
                        var el = {};
                        el['steps'] = pGeneric(val.substring(2-o,8-o));
                        el['utc'] = pUTC(val.substring(8-o,16-o));
                        el['examount'] = pExamount(val.substring(16-o,20-o));
                        el['calories'] = pCalories(val.substring(20-o,28-o));
                        el['t_exercise'] = pGeneric(val.substring(28-o,32-o));
                        el['distance'] = pGeneric(val.substring(32-o,36-o));
                        el['status'] = pStatusDay(val.substring(36-o,38-o));
                        el['voltage'] = pVoltage(val.substring(38-o,40-o));
                        data.push(el);
                        val = val.substring(40-o);
                        o = 2;
                    }
                    response['data'] = data;
                    
                    // battery status
                    var tmp = data[data.length-1];
                    var old = localStorageService.get("battery") || {newVal:{voltage:10.0,level:7}};
                    localStorageService.set("battery", {newVal:{voltage:tmp.voltage, level:tmp.status['battery_level']},oldVal:old.newVal});
                    
                    console.log(JSON.stringify(response));
                    break;
                case "53":
                case "73": 
                case "75":
                    response['utc'] = pUTC(val.substring(2,10));
                    response['utc_offset'] = pUTCOffset(val.substring(10,12));
                    response['unsent_count'] = pGeneric(val.substring(12,16));
                    response['current_count'] = pGeneric(val.substring(16,20));
                    val = val.substring(20);
                    
                    var data = [];
                    var data_size = (cw == "75") ? 4 : 2;
                    while (val.length > 0) {
                        data.push(val.substring(0,data_size));
                        val = val.substring(data_size);
                    }
                    response['data'] = data;
                    
                    console.log(JSON.stringify(response));
                    break;
                case "72":
                    response['pause_times'] = parseInt(val.substring(2,4),16); // range (0,10)
                    response['utc_start'] = parseInt(val.substring(4, 12), 16); 
                    
                    var l = val.length;
                    response['avr_stride_frequency'] = parseInt(val.substring(l-2, l), 16);
                    response['max_stride_frequency'] = parseInt(val.substring(l-4, l-2), 16);
                    response['avr_heart_rate'] = parseInt(val.substring(l-6, l-4), 16);
                    response['max_heart_rate'] = parseInt(val.substring(l-8, l-6), 16);
                    response['total_calories'] = parseInt(val.substring(l-16, l-8), 16);
                    response['total_steps'] = parseInt(val.substring(l-22, l-16), 16);
                    response['total_time_seconds'] = parseInt(val.substring(l-26, l-22), 16); // in s
                    response['utc_stop'] = parseInt(val.substring(l-34, l-26), 16); 
                    
                    val = val.substring(12,l-34);
                    var data = [];
                    while (val.length > 0) {
                        data.push ({
                            utc_pause:val.substring(0,2),
                            utc_rerun:val.substring(2,4)
                        });
                        val = val.substring(4);
                    }
                    response['data'] = data;
                    
                    console.log(JSON.stringify(response));
                    break;
                case "75":
                    response['utc'] = parseInt(val.substring(2,10));
                    response['interval_1'] = parseInt(val.substring(10,14));
                    response['interval_2'] = parseInt(val.substring(14,18));
                    response['interval_3'] = parseInt(val.substring(18,22));
                    break;
                case "67":
                    mambo2_request_parse.parse(val);
                    break;
                default:
                    console.error("Error: Uncought command word in package - "+cw+".");
            }
            
            // Save data
            switch (cw) {
                case "52":                                         
                    console.log("\n\n----------SLEEPDATA------------\n");
                    btd_compress.sleep_data(response_legacy);
                    mambo2_sleep_compress.compress(response);
                    break;
                case "51":
                    console.log("\n\n----------DAYTIMEDATA----------\n");
                    btd_compress.moving_data(response);
                    break;
                case "57":
                    console.log("\n\n----------HOURDATA-------------\n");
                    btd_compress.moving_data(response);
                    break;
                case "53": 
                    console.log("\n\n----------HEARTDATA------------\n");
                    mambo2_compress_data.compress(response, false, "heart");
                    break;
                case "72":
                    var raw = localStorageService.get("raw_running_data") || [];
                    raw.push(response);
                    localStorageService.set("raw_running_data", raw);
                    console.log("\n\n----------RUNNINGDATA------------\n");
                    break;
                case "73":
                    var raw = localStorageService.get("raw_running_heart_data") || [];
                    raw.push(val);
                    localStorageService.set("raw_running_heart_data", raw);
                    console.log("\n\n----------RUNNINGHEARTDATA------------\n");
                    break;
                case "75":
                    var raw = localStorageService.get("raw_heart_rate_interval_data") || [];
                    raw.push(val);
                    localStorageService.set("raw_heart_rate_interval_data", raw);
                    console.log("\n\n----------HEARTINTERVALDATA------------\n");
                    break;
                case "7F":
                    var raw = localStorageService.get("raw_calories_data") || [];
                    raw.push(val);
                    localStorageService.set("raw_calories_data", raw);
                    console.log("\n\n----------CALORIESDATA------------\n");
                    break;
            }
        }
    }
});