angular.module('btdata.services')

.factory('$mambo2Sleep', function($q, mambo2_util, $window, localStorageService){ // localStorage for debug
    function analysis(hexString, utc, utcOffset, debugDate) {
        var tz = moment().format("Z");
        var timezone = parseInt(tz.split(":")[0]);
        
        var plugin = window.cordova.plugins.lssleep;
        var arg = [hexString, utc, utcOffset, timezone]; 

        // create hexID for preventing doubles in storage later
        var hexID = hexString.substring(0,6);
        return $q(function(resolve, reject) {
                plugin.LSSleep(
                    arg,
                    function(success) {
                        // DEBUG
                        debugDate = moment(debugDate).format('YYYY-MM-DD');
                        var debugData = localStorageService.get("debug-data") || {};
                        if (!debugData[debugDate]) debugData[debugDate] = {'raw':null,'raw_combined':null,'parsed_data':null, 'parsed_by_date':null};
                        debugData[debugDate]['parsed_data'] = success;
                        localStorageService.set("debug-data", debugData);
                        console.log("DEBUG "+debugDate+": parsed_data");
                        // DEBUG END
                        
                        
                        for (var i = 0; i < success.length; i++) {
                            success[i]['hexID'] = hexID+"_"+i;
                        }
                        mambo2_util.success("$mambo2Sleep", success);
                        return resolve(success);
                    },
                    function(error) {
                        mambo2_util.error("Error on native sleep analysis.","$mambo2Sleep:analysis", error);
                        return reject(false);
                    }
                )
            });
    }
    
    return {
        analysis:analysis
    }
})

.factory('mambo2_sleep_compress', function(localStorageService, mambo2_sleep_fetch){
    _discard = [];

    // only store raw data, that is still incomplete
    function discardIfFinished(data, date, offset) {
        var comp_date = moment(date).subtract(2,'d').format('YYYY-MM-DD');
        var discard = [];
        var keys = Object.keys(data);
        for (var i = 0, len = keys.length; i < len; i++) {
            if (keys[i] < comp_date) {   // works with format 'YYYY-MM-DD'
                discard.push(keys[i]);
            }
        };
        
        if (discard.length > 0) {
            angular.forEach(discard, function(idx){
                if (_discard.indexOf(idx)<0) {
                    _discard.push(idx);
                    mambo2_sleep_fetch.byDate(idx, offset).then(function(success){
                        var test = localStorageService.get('native_sleep_data') || {};
                        if (test[idx]) {
                            var rawData = localStorageService.get('raw_sleep_data') || {};
                            delete rawData[idx];
                            console.log("Deleted raw sleep data for idx "+idx);
                            localStorageService.set('raw_sleep_data', rawData);
                        } else {
                            console.log("Attempted to delete idx "+idx+", but no result data available!");
                        }
                    });
                }
            });
        } else {
            mambo2_sleep_fetch.byDate(date, offset);
        }
    }
    
    /*
     *  raw_sleep_data:
     *  - format: {<YYYY-MM-DD>:[{utc:<utc>,offset:<utc_offset>,data:<hex_string>},...]}
     */
    
    /*
    
    response['utc'] = pGeneric(val.substring(2,10));
    response['utc_offset'] = pUTCOffset(val.substring(10,12));
    response['data'] = val.substring(20);
    var tmp_unsent = pGeneric(val.substring(12,16));
    if (tmp_unsent > 0) response['unsent_count'] = pGeneric(val.substring(12,16));
    response['count'] = pGeneric(val.substring(16,20));
    
    */
    function compress(data) {
        // fetch raw data and make sure it is sorted
        var rawData = localStorageService.get('raw_sleep_data') || {};
        
        // get index in from utc in format "YYYY-MM-DD"
        var d = moment(data.utc*1000);
        var idx = d.format("YYYY-MM-DD");
        var idx_ref = moment(idx).startOf('day');
        var diff = d.diff(idx_ref,'m');
        if (diff > 839) idx = idx_ref.add(1,'d').format("YYYY-MM-DD"); // // if date is past 1:59pm move to next day
        
        // set check_data for next compress()
        var check_data = localStorageService.get('_sleep_validation') || {};
        data['date'] = idx;
        localStorageService.set('_sleep_validation', data);
        
        // check for unsent
        var added_to_existing = false;
        if (check_data.unsent_count && check_data.unsent_count > 0) {
            if (check_data.utc + (check_data.current_count * check_data.utc_offset) == data.utc) {
                // only combine unsent data if this still starts in that particular day
                if (moment(check_data.date).minutes(840).isAfter(d,'minute')) {
                    // combine unsent data
                    console.log("combine unsent data");
                    var tmp_data = rawData[check_data.date];
                    var tmp_idx = 0;
                    for (var len = tmp_data.length; tmp_idx < len; tmp_idx++) {
                        if (tmp_data[tmp_idx].utc == check_data.utc) break;
                    }
                    rawData[check_data.date][tmp_idx]['data'] += data.data;
                    rawData[check_data.date][tmp_idx]['unsent_count'] -= data.current_count;
                    rawData[check_data.date][tmp_idx]['current_count'] += data.current_count;
                    check_data.data += data.data;
                    check_data.current_count += data.current_count;
                    check_data.unsent_count -= data.current_count;
                    if (check_data.unsent_count <= 0) delete check_data['unsent_count'];
                    localStorageService.set('_sleep_validation', check_data);
                    added_to_existing = true;
                } 
            } 
        }
        
        if (!added_to_existing) {
            var entry = {
                utc:data.utc,
                utc_offset:data.utc_offset,
                data:data.data,
                debug:moment(data.utc*1000).format("YYYY-MM-DD HH:mm")
            };
            console.log("RAW_SLEEP: Added data to raw_sleep_data -> "+JSON.stringify(entry));

            if (!rawData[idx]) rawData[idx] = [entry];
            else {
                // search if there is already an element with that timestamp  
                var push = true;
                var idx_check = rawData[idx];
                for (var i = 0, len = idx_check.length; i < len; i++) {
                    if (idx_check[i].utc == entry.utc) {
                        rawData[idx][i] = entry;
                        push = false;
                        break;
                    }
                }
                if(push) rawData[idx].push(entry);
            }
        }
        
        localStorageService.set('raw_sleep_data', rawData);
        discardIfFinished(rawData, idx, data.utc_offset);
    }
    
    return {
        compress:compress
    }
})

.factory('mambo2_sleep_fetch', function(localStorageService, $mambo2Sleep, $q, mambo2_util, mambo2_heart_data, bthf_add){
    
    function processData(nativeResult, date) {
        // process the incoming sleep analyse results
        var native_sleep_data = localStorageService.get('native_sleep_data') || {};
        // ONLY FOR TESTING -> REMOVE LATER
       // native_sleep_data = {};
        // END! ONLY FOR TESTING -> REMOVE LATER !END 
        
        for (var i= 0; i < nativeResult.length; i++) {
            var data = nativeResult[i];
            var compressedStatus = [];
            
            // change format to compressed format
            angular.forEach(data.sleepStatus, function(val){
                compressedStatus.push({
                    utc_s:val.startTime,
                    m_d:val.duration,
                    status:val.status
                })
            });
            
            var compressedData = {
                c_awake:data.awakeCount,
                m_awake:data.awakeSleepTime,
                m_deep:data.deepSleepTime,
                m_light:data.lightSleepTime,
                utc_getup:data.getupTimeUTC,
                utc_sleep:data.sleepTimeUTC,
                data:compressedStatus,
                hexID:data.hexID
            }
            
            // get index and check if data is available
            var idx = moment(data.getupTimeUTC*1000).format('YYYY-MM-DD');
            if (!native_sleep_data[idx]) native_sleep_data[idx] = [];
            
            // check if there is data for this hexID already
            var found = false;
            for (var j=0,len=native_sleep_data[idx].length; j<len; j++) {
                if (native_sleep_data[idx][j].hexID == compressedData.hexID) {
                    native_sleep_data[idx][j] = compressedData;
                    found = true;
                    console.log("DISCARD DATA: "+idx);
                }
            }
            
            // add to parsed
            if (!found) {
                native_sleep_data[idx].push(compressedData);
                bthf_add.addData('sleep', {data:compressedData.data, date:idx})
            }
            
        }
        localStorageService.set('native_sleep_data', native_sleep_data);
        
        // get results for query date
        var native_sleep_data = localStorageService.get('native_sleep_data') || {};
        var idx = moment(date).format("YYYY-MM-DD");
        var result = native_sleep_data[idx];
        
        // sort by getupTimeUTC
        if (result) {
            result.sort(function(a,b) {
                if (a.utc_getup < b.utc_getup) return -1;
                if (a.utc_getup > b.utc_getup) return 1;
                return 0;
            });
            result[0]['date'] = date;
        }
        
        // DEBUG
        var debugDate = moment(date).format('YYYY-MM-DD');
        var debugData = localStorageService.get("debug-data") || {};
        if (!debugData[debugDate]) debugData[debugDate] = {'raw':null,'raw_combined':null,'parsed_data':null, 'parsed_by_date':null};
        debugData[debugDate]['parsed_by_date'] = result;
        localStorageService.set("debug-data", debugData);
        console.log("DEBUG "+debugDate+": parsed_by_date");
        // DEBUG END
        
        return result;
    }
    
    // NOTE: this function assumes that the offset is always 300
    // -> possible way to use the offset value is to sort data by key:utc_offset, combine each group and send it to native algorithm
    // -> this approach would mean that the promises need to be chained and at last all the results need to be combined
    function byDate(date, offset) {
        // fetch relevant data and combine it
        var d = moment(date);
        var idx = d.format("YYYY-MM-DD");
        var rawData = localStorageService.get('raw_sleep_data') || {};
        var data = rawData[idx] || [];
        
        // last entry from day before
        var tmp_idx = moment(idx).subtract(1,'d').format("YYYY-MM-DD");
        if ((tmp_arr = rawData[tmp_idx])) {
            data.push(tmp_arr[tmp_arr.length-1]);
        }
        
        // data sort by utc
        data.sort(function(a,b){
            if (a.utc < b.utc) return -1;
            if (a.utc > b.utc) return 1;
            return 0;
        });
        
        // DEBUG 
        var debugDate = moment(date).format('YYYY-MM-DD');
        var debugData = localStorageService.get("debug-data") || {};
        if (!debugData[debugDate]) debugData[debugDate] = {'raw':null,'raw_combined':null,'parsed_data':null, 'parsed_by_date':null};
        debugData[debugDate]['raw'] = data;
        localStorageService.set('debug-data', debugData);
        console.log("DEBUG "+debugDate+": raw");
        // DEBUG END
        
        // combine strings
        hexString = "";
        start = 0;
        if (data.length > 0) {
            start = data[0].utc;
            for (var i = 0, len = data.length; i < len; i++) {
                console.log(JSON.stringify(data[i]));
                hexString += data[i].data;
            }
        }
        
        // parse hexstring
        var parsed = "";
        var _hexString = angular.copy(hexString);
        var repeat = undefined;
        while(_hexString.length > 0) {
            var tmp = _hexString.substring(0,2);
            _hexString = _hexString.substring(2);
            var tmpInt = parseInt(tmp, 16);
            
            // catch FFs
            if (tmp == "FF" || tmp == "ff") {
                parsed += "FF";
                continue;
            }
            
            // last value indicated repeat
            if (repeat) {
                while (tmpInt--) {
                    parsed += repeat;
                }
                repeat = undefined;
                continue;
            }
            
            // 8f02
            // check for highest bit being 1
            if (tmpInt >= 0x80) {
                repeat = mambo2_util.padStr((tmpInt - 0x80).toString(16), 2);
            } else {
                parsed += tmp;
            }
        }
        
        
        // TEST -> add heart rate to sleep
        heart_data = localStorageService.get('alt_heart_data') || {};
        var h_data2 = heart_data[date] || {};
        var h_data1_date = moment(date).subtract(1,'d').format('YYYY-MM-DD');
        var h_data1 = heart_data[h_data1_date] || {};
        if (h_data1 || h_data2) {
            // concatenate objects 
            for (key in h_data2) {
                if (h_data2.hasOwnProperty(key)) {
                    _key = (parseInt(key)+1440).toString();
                    h_data1[_key] = h_data2[key];
                }
            }
            
            // get the start as minute ref
            var m_tmp = moment((start+1)*1000);
            var m_tmp2 = angular.copy(m_tmp);
            var m_start = m_tmp.diff(m_tmp2.startOf('day'), 'm');
            
            var test = localStorageService.get('test') || {'greater_than_day':8,'greater_than_night':2,'less_than_night':2};
            localStorageService.set('test', test);
            
            var _at_rest = mambo2_heart_data.heartRateAtRest(date); 
            at_rest_night = _at_rest.night ? parseInt(_at_rest.night.val) : parseInt(_at_rest.all.val);
            at_rest_day = _at_rest.day ? parseInt(_at_rest.day.val) : parseInt(_at_rest.all.val);
            if (at_rest_night) {
                var result = "";
                var _parsed = angular.copy(parsed);
                var ref = moment(date).subtract(1,'d').startOf('day').add(m_start, 'm');
                while (_parsed.length > 0) {
                    var tmp = _parsed.substr(0,2);
                    if ((value = h_data1[m_start])) {
                        var diff_night = value-at_rest_night;
                        var diff_day = value-at_rest_day;
                        if (diff_day > 0) {
                            var tmp_val = parseInt(tmp,16) + (diff_day * test.greater_than_day);
                            tmp_val = tmp_val < 91 ? tmp_val : 90;            // never more than 2 chars
                            tmp = mambo2_util.padStr((tmp_val).toString(16),2); // never less than 2 chars
                        } else if (diff_night > 0) {
                            var tmp_val = parseInt(tmp,16) + (diff_night * test.greater_than_night);
                            tmp_val = tmp_val < 91 ? tmp_val : 90;            // never more than 2 chars
                            tmp = mambo2_util.padStr((tmp_val).toString(16),2); // never less than 2 chars
                        } else {
                            var tmp_val = parseInt(tmp,16) + (diff_night * test.less_than_night);
                            tmp_val = tmp_val < 91 ? tmp_val : 90;              // never more than 2 chars
                            tmp_val = tmp_val >= 0 ? tmp_val : 0; 
                            tmp = mambo2_util.padStr((tmp_val).toString(16),2); // never less than 2 chars
                        }
                    }

                    result += tmp;
                    _parsed = _parsed.substr(2);
                    m_start += 5;
                }
                
                if (result.length != parsed.length) console.error("Heart rate was not added correctly on raw sleep data!");
                else {
                    console.log("Modified Sleep Data:\nold: "+parsed+"\nnew: "+result); 
                    parsed = result;
                }
            }
        }
        
        
       
        // DEBUG
        debugData[debugDate]['raw_combined'] = parsed;
        console.log("DEBUG "+debugDate+": raw_combined");
        localStorageService.set('debug-data', debugData);
        // DEBUG END
        
        // use algorithm to get real sleep data
        if (parsed.length != 0) {    
            return $mambo2Sleep.analysis(parsed, start, offset, date).then( // date for debug
                function(success) {
                    return processData(success, date);
                },function(error) {
                    return error;
                }
            );
        } else {
            return $q(function(resolve){
                return resolve(false);
            })
        }
    }
    
    return {
        byDate:byDate
    }
})

.factory('mambo2_sleep_process', function(){
    function process(sleepAnalyseResults) {
        
    }
    
    return {
        process:process
    }
});