'use strict';

angular.module('bthealthfit', []).factory('bthf_constants', function () {
    var _timeOffset = void 0;

    return {
        isAndroid: ionic.Platform.isAndroid(),
        bundle: "com.tobiasphilipp.leicke",
        name: "smartlife",
        sourceBundleId: "com.leicke.smartlife",
        timeOffset: function timeOffset() {
            if (_timeOffset) return _timeOffset;else {
                var tz = moment().format("Z");
                var _tz = parseInt(tz.split(":")[0]) + parseInt(tz.split(":")[1]) * 0.1;
                var offset = parseInt(_tz * 60 * 60 * 1000);
                _timeOffset = offset;
                return offset;
            }
        }
    };
}).factory('bthf_init', function (localStorageService, $mdDialog, $ionicPlatform, $rootScope) {
    var bthf = {
        enabled: false,
        authenticated: false,
        shealth_initialized: false,
        auth_prompt: function auth_prompt() {
            if (ionic.Platform.isAndroid()) {
                bthf.auth();
            } else if (ionic.Platform.isIOS()) {
                if (!localStorageService.get("use_health_prompt")) {
                    var usagePr = $mdDialog.confirm({
                        title: 'Apple Health',
                        textContent: 'Diese App unterstützt den Datenaustausch mit Apple Health. Möchten Sie diesen jetzt einrichten?',
                        ok: 'ja',
                        cancel: 'später'
                    });
                    $mdDialog.show(usagePr).then(function (success) {
                        localStorageService.set("use_health_prompt", true);
                        bthf.auth();
                    }, function () {
                        localStorageService.set("use_health_prompt", true);
                    });
                } else bthf.auth();
            }
        },
        auth: function auth() {
            var datatypes = ['calories', 'distance', 'steps', 'heart_rate']; // , 'activity'
            // TODO: Android quirks -> https://github.com/dariosalvi78/cordova-plugin-health
            navigator.health.requestAuthorization(datatypes, function (success) {
                console.log(JSON.stringify("HF_success auth: " + success));
                navigator.health.isAuthorized(datatypes, function (success) {
                    if (success) {
                        bthf.authenticated = true;
                        bthf.enabled = true;
                    }
                }, function (error) {
                    console.log(JSON.stringify("HF_error is_auth: " + error));
                });
            }, function (error) {
                console.log(JSON.stringify("HF_error auth: " + error));
            });
        },
        prompt: function prompt() {
            if (ionic.Platform.isAndroid()) {
                var installPr = $mdDialog.confirm({
                    title: 'GoogleFit',
                    textContent: 'Diese App unterstützt den Datenaustausch mit GoogleFit. Möchten Sie GoogleFit jetzt installieren?',
                    ok: 'installieren',
                    cancel: 'später'
                });
                $mdDialog.show(installPr).then(function (success) {
                    localStorageService.set("install_health_prompt", true);
                    navigator.health.promptInstallFit(function (success) {
                        console.log(JSON.stringify("HF_success: " + success));
                        bthf.auth();
                    }, function (error) {
                        console.log(JSON.stringify("HF_error promt: " + error));
                    });
                }, function () {
                    localStorageService.set("install_health_prompt", true);
                });
            } else if (ionic.Platform.isIOS()) {
                // TODO: check how this all needs to be setup in iOS
            }
        },
        init: function init() {
            if (!bthf.enabled) $ionicPlatform.ready(function () {
                navigator.health.isAvailable(function (success) {
                    if (!success) {
                        var pr = localStorageService.get("install_health_prompt");
                        if (!pr) {
                            console.log("Prompt now!");
                            bthf.prompt();
                        } else {
                            bthf.auth();
                        }
                    } else {
                        bthf.auth_prompt();
                    }
                }, function (error) {
                    console.log(JSON.stringify("HF_error available: " + error));
                });
            });
        },
        disable: function disable() {
            bthf.enabled = false;
        },
        init_shealth: function init_shealth() {
            if (!bthf.shealth_initialized) $ionicPlatform.ready(function () {
                // SHealth
                if (cordova.plugins.shealth) {
                    cordova.plugins.shealth.init(null, function (success) {
                        console.log("SH_success SHealth init: " + JSON.stringify(success));
                        if (success.status && success.status === 1) {
                            console.log("SH_success SHealth init: all set :)");
                            bthf.shealth_initialized = true;
                        }
                    }, function (error) {
                        console.log("SH_error SHealth init: " + JSON.stringify(error));
                        if (error.status && error.status === 0) {
                            console.log("Following error to handle -> " + error.type + ": " + error.message);
                            switch (error.type) {
                                case PLATFORM_NOT_INSTALLED:
                                    // prompt install
                                    break;
                                case OLD_VERSION_PLATFORM:
                                case PLATFORM_DISABLED:
                                case USER_AGREEMENT_NEEDED:
                                    // alert that something is wrong with samsung health setup. 
                                    break;
                                default:
                                    // default
                                    break;
                            }
                        }
                    });
                }
            });
        },
        disable_shealth: function disable_shealth() {
            console.warn("There is currently no disable function for Samsung Health.");
        }
    };

    return bthf;
});