'use strict';

angular.module('bthealthfit').factory('bthf_add', function (localStorageService, bthf_constants, bthf_init, $q, $timeout) {
    var bthf = {
        addData: function addData(type, data) {
            if (bthf_init.authenticated && bthf_init.enabled) switch (type) {
                case 'steps':
                case 'calories':
                case 'distance':
                    bthf.receiveMovingData(data, type);
                    break;
                case 'heart':
                    bthf.heartRate(data);
                    break;
                case 'sleep':
                    bthf.sleep(data.data, data.date);
                    break;
                default:
                    console.warn("bthf_add:addData() unknown type '" + type + "'!");
                    break;
            }
        },
        heartRate: function heartRate(data) {
            var params = {
                startDate: moment(data.date).add(data.offset_m - data.utc_offset, 'm').toDate(),
                endDate: moment(data.date).add(data.offset_m, 'm').toDate(),
                dataType: 'heart_rate',
                sourceName: bthf_constants.name,
                sourceBundleId: bthf_constants.bundle,
                value: data.value
            };
            navigator.health.store(params, function () {
                console.log("Heart rate was successfully stored.");
            }, function (s_error) {
                console.error("HF store heart rate error: " + JSON.stringify(s_error));
            });
        },
        sleep: function sleep(data, date) {
            for (var i = 0; i < data.length; i++) {
                var val = data[i];

                // change format to compressed format
                var params = {
                    startDate: moment(val.utc_s * 1000).toDate(),
                    endDate: moment(val.utc_s * 1000).add(val.m_d, 'm').toDate(),
                    dataType: 'activity',
                    sourceName: bthf_constants.name,
                    sourceBundleId: bthf_constants.bundle
                };

                switch (val.status) {
                    case 1:
                        params['value'] = 'sleep.awake';
                        break;
                    case 2:
                        params['value'] = 'sleep.light';
                        break;
                    case 3:
                        params['value'] = 'sleep.deep';
                        break;
                    default:
                        console.error("Unknown sleep status!");
                        break;
                }

                navigator.health.store(params, function () {
                    console.log("Sleep was successfully stored.");
                }, function (s_error) {
                    console.error("HF store sleep error: " + JSON.stringify(s_error));
                });
                /*
                    navigator.health.query({
                        startDate:params.startDate,
                        endDate:params.endDate,
                        dataType:'activity'
                    }, function(success) {
                        // check if data is duplicate
                         let s = moment(params.startDate);
                        let e = moment(params.endDate);
                        let found = false;
                        for (let i = 0; i < success.length; i++) {
                            let tmp = success[i];
                            if (tmp.sourceBundleId == bthf_constants.sourceBundleId) {
                                if (moment(tmp.startDate).isSame(s) && moment(tmp.endDate).isSame(e)) {
                                    found = true;
                                    break;
                                }
                            }
                        }
                        
                        if (!found) {
                            console.log("HF store sleep log: "+JSON.stringify(params));
                            navigator.health.store(params, function() {
                                console.log("Sleep was successfully stored.");
                            }, function(s_error){
                                console.error("HF store sleep error: "+JSON.stringify(s_error));
                            });
                        }
                    }, function(q_error) {
                        console.log("HF query sleep error: "+JSON.stringify(q_error));
                        // data might just not be there. good - now save data
                        navigator.health.store(params, function() {
                            console.log("Sleep was successfully stored.");
                        }, function(s_error){
                            console.error("HF store sleep error: "+JSON.stringify(s_error));
                        });
                    });
                */
            }
        },
        moving_data_dates: [],
        moving_data_cache: { steps: [], calories: [], distance: [] },
        moving_data_timeout: { steps: null, calories: null, distance: null },
        receiveMovingData: function receiveMovingData(_data, type) {
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = _data[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var data = _step.value;

                    console.log("HF " + type + " Incoming: " + JSON.stringify(data));
                    bthf.moving_data_cache[type].push(data);
                    bthf.moving_data_timeout[type] = $timeout(function () {
                        var tmp_cache = bthf.moving_data_cache[type];
                        bthf.moving_data_cache[type] = [];
                        if (tmp_cache.length > 1) {
                            tmp_cache.sort(function (a, b) {
                                if (!a.data || !a.data.time) console.error("HF A.DATA: " + JSON.stringify(a));
                                if (!b.data || !b.data.time) console.error("HF B.DATA: " + JSON.stringify(b));
                                return a.data.time - b.data.time;
                            });
                        }

                        console.log("HF ORDERED MOVING: " + JSON.stringify(tmp_cache));
                        var _iteratorNormalCompletion2 = true;
                        var _didIteratorError2 = false;
                        var _iteratorError2 = undefined;

                        try {
                            for (var _iterator2 = tmp_cache[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                                var d = _step2.value;

                                bthf.addMovingData(d, type);
                            }
                        } catch (err) {
                            _didIteratorError2 = true;
                            _iteratorError2 = err;
                        } finally {
                            try {
                                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                                    _iterator2.return();
                                }
                            } finally {
                                if (_didIteratorError2) {
                                    throw _iteratorError2;
                                }
                            }
                        }
                    }, 10000);
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            ;
        },
        addMovingData: function addMovingData(d, type) {
            var t = type.substr(0, 1);
            var lastHFSent = localStorageService.get('lastHFSent_moving') || {};
            if (!lastHFSent[t]) lastHFSent[t] = {};

            var _d = d.data;
            var _endDate = moment(d.date).startOf('day').add(_d.time, 's');

            var data = {};

            var moving_dates = localStorageService.get('moving_data_health_dates') || [];
            if (lastHFSent[t].date != d.date && moving_dates.indexOf(moment(d.date).format('YYYY-MM-DD')) < 0) {
                moving_dates.push(moment(d.date).format('YYYY-MM-DD'));
                localStorageService.set('moving_data_health_dates', moving_dates);

                if (!bthf_constants.isAndroid) {
                    // no previous data sent for this day 
                    data['startDate'] = moment(d.date).toDate();
                } else {
                    var nsd = localStorageService.get('native_sleep_data');
                    var night = nsd[moment(d.date).format('YYYY-MM-DD')] || false;
                    if (night) {
                        var _startDate = moment(d.date).add('m', night.utc_getup).startOf('hour').add(1, 'h');
                        if (_startDate.isBefore(_endDate)) data['startDate'] = _startDate.toDate();else data['startDate'] = _endDate.subtract(5, 'm').toDate();
                    } else {
                        data['startDate'] = moment(d.date).toDate();
                        /* FIRST TEST IF SLEEP IS ENOUGH
                        let md = localStorageService.get('moving_data');
                        let night = md[moment(d.date).format('YYYY-MM-DD')];
                        let found = false;
                        for (let tmp of night) {
                            if (tmp.time < _d.time) {
                                if (!found || tmp.time > found) {
                                    found = tmp.time;
                                } 
                            }
                        }
                        if (found) {
                            // TODO:
                        }
                        */
                    }
                }
                data['value'] = _d[t];
            } else {
                // previously sent data on this day
                data['startDate'] = moment(lastHFSent[t].date).add(lastHFSent[t].time, 's').toDate();
                data['value'] = _d[t] - lastHFSent[t].value;
            }
            data['endDate'] = _endDate.toDate();
            data['dataType'] = type;
            data['sourceName'] = bthf_constants.name;
            data['sourceBundleId'] = bthf_constants.bundle;

            if (data.value > 0) {
                console.log("HF store " + type + " log: " + JSON.stringify(data));
                lastHFSent[t] = {
                    date: d.date,
                    time: _d.time,
                    value: _d[t]
                };
                localStorageService.set('lastHFSent_moving', lastHFSent);
                var _tmp = localStorageService.get('tmp_Moving_LIST') || [];
                if (t == 's') _tmp.push(lastHFSent[t]);
                localStorageService.set('tmp_Moving_LIST', _tmp);

                navigator.health.store(data, function () {
                    console.log(type + " was successfully stored: " + JSON.stringify(data));
                }, function (s_error) {
                    console.error("HF store " + type + " error: " + JSON.stringify(s_error));
                });
            }
        }
    };

    return bthf;
});