"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var MessageHelper = function () {
    function MessageHelper($cordovaBluetoothLE) {
        _classCallCheck(this, MessageHelper);

        this.$cordovaBluetoothLE = $cordovaBluetoothLE;
    }

    _createClass(MessageHelper, [{
        key: "encodedToByteString",
        value: function encodedToByteString(input) {
            var val = this.$cordovaBluetoothLE.encodedStringToBytes(input);
            var byteStr = "";
            for (var i = 0; i < val.length; i++) {
                var byte = val[i].toString(16);
                if (byte.length == 1) byte = "0" + byte;
                byteStr += byte;
            }
            return byteStr;
        }
    }, {
        key: "addBytes",
        value: function addBytes(input) {
            var result = 0;
            var arr = input.match(/.{1,2}/g);
            for (var i = 0; i < arr.length; i++) {
                result += parseInt(arr[i], 16);
            }
            return result.toString(16);
        }
    }, {
        key: "pad",
        value: function pad(num, size) {
            var s = num + "";
            while (s.length < size) {
                s = "0" + s;
            }return s;
        }
    }, {
        key: "length",
        value: function length(input) {
            return ((input.length + 4) / 2).toString(16);
        }
    }, {
        key: "convertToHex",
        value: function convertToHex(str) {
            var hex = '';
            for (var i = 0; i < str.length; i++) {
                hex += '' + pad(str.charCodeAt(i).toString(16), 2);
            }
            return hex;
        }
    }, {
        key: "encodeValue",
        value: function encodeValue(input) {
            var bytes = hexStringToBinary(input);
            return this.$cordovaBluetoothLE.bytesToEncodedString(bytes);
        }
    }, {
        key: "hexStringToBinary",
        value: function hexStringToBinary(hexString) {
            if (hexString == null || hexString == "") {
                return null;
            }
            hexString = hexString.toUpperCase();
            var length = hexString.length / 2;
            var hexChars = hexString.split('');
            var d = new Array(length);
            for (var i = 0; i < length; i++) {
                var pos = i * 2;
                d[i] = parseInt(charToByte(hexChars[pos]) << 4 | charToByte(hexChars[pos + 1]), 10) & 255;
            }
            return d;
        }
    }]);

    return MessageHelper;
}();

angular.module('btmessage', ['ionic', 'ngCordova', 'LocalStorageModule', 'ngCordovaBluetoothLE', 'btconnect']).factory('bt_messenger', function (bt_message, $cordovaBluetoothLE) {
    var send = function send(message, address) {
        var hexInput = bt_message.generateMessage(message);
        var params = {
            "value": hexInput,
            "service": "FCC0",
            "characteristic": "FCC6",
            "address": address
        };

        $cordovaBluetoothLE.write(params).then(function (success) {
            console.log("Message-Write Success: " + JSON.stringify(success));
        }, function (error) {
            console.log("Message-Write Error: " + JSON.stringify(error));
        });
    };

    return {
        send: send
    };
}).factory('bt_sms_messenger', function (bt_message, $cordovaBluetoothLE, mambo2_services) {
    var mh = new MessageHelper($cordovaBluetoothLE);
    var idx = 0;
    var message = undefined;
    var address = undefined;
    var content = "";
    var c_idx = 0;
    var is_content = false;

    function subscribe(address) {
        var params = {
            "service": "FCC0",
            "characteristic": "FCC7",
            "address": address
        };

        console.log("SUBSCRIBE FCC7");
        return $cordovaBluetoothLE.subscribe(params).then(null, function (error) {
            console.error("Error subscribing to FCC7 in bt_sms_messenger:subscribe():" + JSON.stringify(error));
            if (error.error === "subscription") {
                $cordovaBluetoothLE.unsubscribe(params).then(function (success) {
                    console.log("Unsubscribe to FCC7 (-> subscribe again):" + JSON.stringify(success));
                    subscribe(address);
                }, function (error) {
                    console.error("Subscription to FCC7 failed in bt_sms_messenger:subscribe():" + JSON.stringify(error));
                    q.reject(error);
                });
            }
        }, function (success) {
            if (success.value) {
                var decoded = mh.encodedToByteString(success.value);
                console.log("DECODED FCC7 VALUE: " + decoded);
                var result = bt_message.parseSubscribeResult(decoded);
                console.log("Parsed Decoded FCC7 VALUE: " + JSON.stringify(result));
                if (result.sum_check) {
                    var send_arr = bt_message.generateSmsContentMessage(content, result.type_1, result.message_id);
                    content = send_arr;
                    c_idx = 0;
                    console.log(JSON.stringify(content));
                    send_content();
                } else {
                    console.error("Checksum on message was not correct. Message will not be displayed.");
                }
            } else {
                console.log("Successfully subscribed to FCC7.");
            }
        });
    }

    function send_message() {
        if (idx >= message.length) return;

        var params = {
            "value": message[idx],
            "service": "FCC0",
            "characteristic": "FCC6",
            "address": address
        };

        $cordovaBluetoothLE.write(params).then(function (success) {
            console.log("message-write success: " + JSON.stringify(success));
            idx++;
            console.log("CALL: Inner send_message!");
            send_message();
        }, function (error) {
            console.error("message-write error: " + JSON.stringify(error));
        });
    }

    function send_content() {
        if (c_idx >= content.length) {
            mambo2_services.resume();
            return;
        }

        var params = {
            "value": content[c_idx],
            "service": "FCC0",
            "characteristic": "FCC6",
            "address": address
        };
        $cordovaBluetoothLE.write(params).then(function (success) {
            console.log("content-write success: " + JSON.stringify(success));
            c_idx++;
            send_content();
        }, function (error) {
            console.error("content-write error: " + JSON.stringify(error));
            mambo2_services.resume();
        });
    }

    function send_message_loop(s_message, s_address, s_content) {
        idx = 0;
        message = bt_message.generateSmsTitleMessage(s_message);
        address = s_address;
        content = s_content;

        console.log("CALL: initial send_message_loop.");
        mambo2_services.pause().then(function (success) {
            if (success) send_message(message[idx]);
        });
    }

    return {
        subscribe: subscribe,
        send: send_message_loop
    };
}).factory('bt_message', function ($cordovaBluetoothLE, $q) {
    var mh = new MessageHelper($cordovaBluetoothLE);
    var sms_count = 0;

    function parseSubscribeResult(input) {
        var result = {};
        result['index'] = parseInt(input.substring(0, 2), 16) - 1;
        var total_length = parseInt(input.substring(2, 4), 16) * 2 + 4;
        result['command_id'] = input.substring(4, 6);
        // 01 0b a1 04 00000009 03 0064 0115 00000000000000
        switch (parseInt(result.command_id, 16)) {
            case parseInt("a1", 16):
                // request for sms message
                result['type_1'] = parseInt(input.substring(6, 8));
                result['message_id'] = parseInt(input.substring(8, 16));
                var tmp = input.substring(16, total_length - 4);
                while (tmp.length > 0) {
                    var request_type = tmp.substring(0, 2);
                    var request_size = parseInt(tmp.substring(2, 6), 16);
                    switch (parseInt(request_type)) {
                        case 3:
                            result['content_length'] = request_size;
                            break;
                        default:
                            console.warn("Unknown request for part of message.");
                            break;
                    }
                    tmp = tmp.substring(6);
                }
                break;
            case parseInt("04", 16):
                // command confirmation
                delete result['command_id'];
                result['type_1'] = input.substring(4, 6);
                result['type_2'] = input.substring(6, 8);
                result['message_id'] = parseInt(input.substring(8, 16), 16);
                result['confirmation'] = new Boolean(parseInt(input.substring(16, 18), 16));
                break;
            default:
                console.warn("Unknown FCC7 subscription result: " + input);
        }
        var added_sum = parseInt(mh.addBytes(input.substring(4, total_length - 4)));
        var check_sum = parseInt(input.substring(total_length - 4, total_length));
        result['sum_check'] = added_sum == check_sum;
        return result;
    }

    return {
        // a message of 14 characters max 
        // spaces are removed "H a l l o" -> "Hallo"
        // vibrates 3 times --> not good for showing messages
        generateMessage: function generateMessage(input) {
            input = mh.convertToHex(input);
            // ONLY FOR INCOMING CALL
            input = "0100" + input;
            var output = "01" + mh.pad(mh.length(input), 2) + input + mh.pad(mh.addBytes(input), 4);
            return mh.encodeValue(output);
        },
        generateSmsTitleMessage: function generateSmsTitleMessage(input) {
            // 011B 04 00 0000 01 00000001 000E 546F6269617320
            // 01 0f 04 00 0000 00 0006546f62696173026c

            var type = "04"; // social (=message)
            var action = "00"; // add
            var app_id = "0000"; // SMS
            // TODO:should be unread, but then we would need to get information on unread sms from phone -> look into available info from SMSManager
            sms_count++;
            var count = mh.pad(sms_count.toString(16), 2); // received sms 
            var message_id = mh.pad(sms_count.toString(16), 8); // internal message id
            var title = input; // title will be either number or name of sender
            var title_length = mh.pad(title.length.toString(16), 4); // title length
            var title_hex = mh.convertToHex(title); // title hex string

            var message_content = app_id + count + message_id + title_length + title_hex;

            var message = type + action + message_content;
            var output = mh.pad(mh.length(message), 2) + message + mh.pad(mh.addBytes(message), 4);

            var idx = 0;
            var result = [];
            while (output.length > 0) {
                idx++;
                var val = mh.pad(idx.toString(16), 2) + output.substr(0, 38);
                console.log("MESSAGE " + idx + ": " + val);
                output = output.substr(38);
                result.push(mh.encodeValue(val));
            }

            return result;
        },
        generateSmsContentMessage: function generateSmsContentMessage(content, t, m_id) {
            var len = "00";
            var command_id = "c1";
            var type = mh.pad(t.toString(16), 2);
            var message_id = mh.pad(m_id.toString(16), 8);
            var frame_count = "01";
            var current_idx = "01";

            var message = "03" + mh.pad(content.length.toString(16), 4) + mh.convertToHex(content);
            console.log("MESSAGE-TEXT: " + mh.convertToHex(content));

            var _l = (command_id + type + message_id + frame_count + current_idx + message).length / 2;
            if (_l % 2 != 0) console.error("SMS content error. Command word length not multiple of 2.");
            len = mh.pad((_l + 2).toString(16), 2);

            _l += 1; // add length byte to count
            frame_count = mh.pad(Math.ceil(_l / 19).toString(16), 2);

            var result = [];

            // TODO: Now messages are cut short to 100 chars. If we want to support larger sms we need to send multiple packages.
            frame_count = "01";
            current_idx = "01";

            var all = len + command_id + type + message_id + frame_count + current_idx + message + mh.pad(mh.addBytes(command_id + type + message_id + frame_count + current_idx + message), 4);

            for (var i = 0, _len = Math.ceil(all.length / 38); i < _len; i++) {
                var tmp = all.substring(0, 38);
                tmp = mh.pad((i + 1).toString(16), 2) + tmp;
                result.push(mh.encodeValue(tmp));
                console.log("CONTENT " + (i + 1).toString() + ": " + tmp);
                all = all.substring(38);
            }
            return result;
        },
        generateConfirmation: function generateConfirmation(success) {
            // 0105C0000401
            var len = "05";
            var command_id = "C0";
            var info_type = "04";
            var test = "00";
            var confirm = success ? "00" : "00";
            var word = command_id + info_type + test + confirm;
            var response = "01" + len + word + mh.addBytes(word);
            return response;
        },
        parseResponse: function parseResponse(response) {

            // MESSAGE 1: 01130400000001000000010006546f6269617302
            // MESSAGE 2: 026e
            // resp: 01 13  04  00  0000    01  00000001 0006 546f6269617302
            var byteStr = mh.encodedToByteString(response);
            result = {};
            result['index'] = parseInt(byteStr.substring(0, 2), 16) - 1;
            var total_length = parseInt(byteStr.substring(2, 4), 16);
            result['info1'] = byteStr.substring(4, 6);
            result['info2'] = byteStr.substring(6, 8);
            result['message_type'] = byteStr.substring(8, 12);
            result['message_count'] = byteStr.substring(12, 14);
            result['message_id'] = byteStr.substring(14, 22);
            var m_length = parseInt(byteStr.substr(22, 26), 16) * 2;
            result['message_length'] = m_length;
            result['message_content'] = byteStr.substr(26, m_length);
            result['multiple'] = false;
            if (26 + m_length > 36) {
                result['multiple'] = true;
                result['check_sum'] = byteStr.substring(26 + m_length);
            } else {
                var check_sum = byteStr.substring(26 + m_length);
                var test_sum = mh.addBytes(byteStr.substring(4, 26 + m_length));
                console.log("CHECKSUM: " + check_sum + " vs. TEST: " + mh.pad(test_sum, 4));
                result['sum_check'] = check_sum == mh.pad(test_sum, 4);
            }
            console.log("RESULT: " + JSON.stringify(result));
            return result;
        },
        parseSubscribeResult: parseSubscribeResult
    };
});