'use strict';

/*
 *  request.js
 *  (factory)mambo2_request: request data stored on device
 */

angular.module('btrequest').factory('mambo2_request', function (mambo2_message_sender) {
    function send(type) {
        var cw = "66";
        var value = cw + type;
        mambo2_message_sender.send("8000", value);
    }

    return {
        send: send
    };
}).factory('mambo2_request_parse', function (mambo2_util, $translate, $q, mambo2_message_sender) {

    /*
     *  Helpers
     */

    // requires string separated by a dot. e.g.: "75.38"kg 
    var hexToFloat = function hexToFloat(dec, val) {
        var v = parseInt(val, 16).toString(10);
        var d = "";
        switch (dec.length) {
            case 1:
                d = v.length - (16 - parseInt(dec, 16));
                break;
            case 2:
                d = v.length - (256 - parseInt(dec, 16));
                break;
            case 3:
                d = v.length - (4096 - parseInt(dec, 16));
                break;
        }
        return v.insert(".", d);
    };

    function targetState(input) {
        switch (input) {
            case "01":
                return "steps";
                break;
            case "02":
                return "calories";
                break;
            case "03":
                return "distance";
                break;
            case "04":
                return "exercice amount";
                break;
        }
    }

    function floatOrNot(input) {
        if (parseInt(input.substring(0, 2), 16) > 240) {
            // 240 -> "F0"
            return hexToFloat(input.substring(0, 2), input.substring(2));
        } else return parseInt(input, 16);
    }

    function repetitionDay(input) {
        var byteStr = mambo2_util.padStr(parseInt(input, 16).toString(2), 8);
        var byteArr = byteStr.split("");
        byteArr.splice(0, 1);
        return byteArr.map(function (numStr) {
            return numStr == "1" ? true : false;
        }).reverse();
    }

    /*
     *  Parse
     */

    function userinfo(info) {
        return {
            weight: hexToFloat(info.substring(0, 2), info.substring(2, 8)),
            height: hexToFloat(info.substring(8, 10), info.substring(10, 16)),
            target_cat: targetState(info.substring(16, 18)),
            target_val: floatOrNot(info.substring(18, 20), info.substring(20, 26))
        };
    }

    // modified to return object instead of array of objects -> that won't work with btrequest.html
    function alarmClock(info) {
        console.log("ALARMSTRING: " + info);
        var tmp = {};
        tmp["switch"] = parseInt(info.substring(0, 2), 16).boolValue();
        tmp["count"] = parseInt(info.substring(2, 4), 16);
        var rest = info.substring(4);
        var arr = [rest.substring(0, 18), rest.substring(18, 36), rest.substring(36, 54)];
        tmp["alarms"] = [];
        angular.forEach(arr, function (a) {
            var alarm = {};
            alarm["serial"] = a.substring(0, 2);
            alarm["alarm_on"] = parseInt(a.substring(2, 4), 16).boolValue();
            alarm["hh"] = mambo2_util.padStr(parseInt(a.substring(4, 6), 16), 2);
            alarm["mm"] = mambo2_util.padStr(parseInt(a.substring(6, 8), 16), 2);
            alarm["repeat_array"] = repetitionDay(a.substring(8, 10));
            alarm["settings"] = {
                vibe: a.substring(10, 12),
                length: a.substring(12, 14),
                strength: a.substring(14, 18)
            };
            tmp.alarms.push(alarm);
        });
        return tmp;
    }

    function parse(response) {
        var type = response.substring(2, 4);
        var info = response.substring(4);

        var result = { type: type };
        switch (type) {
            case "01":
                result['data'] = userinfo(info);
                break;
            case "02":
                result['data'] = alarmClock(info);
                break;
        }
        this.data = result;

        // let mambo2 know that everything is fine
        mambo2_message_sender.send("8000", "6701");
    }

    return {
        data: {},
        parse: parse
    };
});