'use strict';

/*
 *  screen.js
 *  (factory)mambo2_screen: initialize and manage screen settings
 *  settings:
 *      - screens order/display
 *      - screen orientation
 */

angular.module('btscreen').factory('mambo2_screen', function (localStorageService, mambo2_util, mambo2_message_sender, mambo2_services, $rootScope) {
    var initiated = false;
    var eventListener;
    function init() {
        var screenSettings = localStorageService.get("screen_settings");
        if (!screenSettings || screenSettings['screens'][0]['name'].indexOf('CUST') >= 0) {
            if (!initiated) eventListener = $rootScope.$on('newData', defaultSettings);
        } else {
            initiated = true;
        }
    }

    function defaultSettings() {
        eventListener();
        var screenSettings = {};
        // no settings found -> initiate 'default' settings
        mambo2_services.pause().then(function (success) {
            if (success) {
                // set default screens;
                var defaultScreens = [{
                    hex: "00",
                    name: "Uhrzeit_Datum",
                    active: true
                }, {
                    hex: "01",
                    name: "Schritte",
                    active: true
                }, {
                    hex: "02",
                    name: "Kalorien",
                    active: true
                }, {
                    hex: "03",
                    name: "Distanz",
                    active: true
                }, {
                    hex: "04",
                    name: "Puls",
                    active: true
                }];
                //,{
                //                        hex: "05",
                //                        name: "Lauf_Sprint",
                //                        active: false
                //                }
                saveScreens(defaultScreens);
                screenSettings['screens'] = defaultScreens;

                // set default orientation -> vertical
                saveOrientation(false);
                screenSettings['orientation'] = "vertical";

                // save settings and resume services
                localStorageService.set('screen_settings', screenSettings);
                mambo2_services.resume();
            } else {
                mambo2_util.error('Error on screen settings init.', 'mambo2_screen:init()');
            }
        });
    }

    function saveScreens(screens) {
        var cw = "7E";
        var length = screens.length;
        var scr = "";
        angular.forEach(screens, function (s) {
            if (s.active) scr += s.hex;else length--;
        });
        length = mambo2_util.padStr(length.toString(16), 2);
        var word = cw + length + scr;
        mambo2_message_sender.send("8000", word);
    }

    function saveOrientation(horizontal) {
        var cw = "7D";
        var orientation = horizontal ? "00" : "01";
        var word = cw + orientation;
        mambo2_message_sender.send("8000", word);
    }

    return {
        init: init,
        saveScreens: saveScreens,
        saveOrientation: saveOrientation
    };
});